@@ -41,9 +41,19 @@
 #include <ipmitool/helper.h>
 #include <ipmitool/ipmi_cc.h>
 
+#if HAVE_CONFIG_H
+# include <config.h>
+#endif
 
 #define IPMI_BUF_SIZE 1024
 
+#if HAVE_PRAGMA_PACK
+#define ATTRIBUTE_PACKING
+#else
+#define ATTRIBUTE_PACKING __attribute__ ((packed))
+#endif
+
+
 /* From table 13.16 of the IPMI v2 specification */
 #define IPMI_PAYLOAD_TYPE_IPMI               0x00
 #define IPMI_PAYLOAD_TYPE_SOL                0x01
@@ -138,6 +148,7 @@ struct ipmi_rq_entry {
 	uint8_t rq_seq;
 	uint8_t *msg_data;
 	int msg_len;
+	int bridging_level;
 	struct ipmi_rq_entry *next;
 };
 
@@ -228,8 +239,9 @@ struct ipmi_rs {
 #define IPMI_NETFN_APP			0x6
 #define IPMI_NETFN_FIRMWARE		0x8
 #define IPMI_NETFN_STORAGE		0xa
-#define IPMI_NETFN_TRANSPORT		0xc
+#define IPMI_NETFN_TRANSPORT	0xc
 #define IPMI_NETFN_PICMG		0x2C
+#define IPMI_NETFN_DCGRP		0x2C
 #define IPMI_NETFN_ISOL			0x34
 #define IPMI_NETFN_TSOL			0x30
 
@@ -239,16 +251,35 @@ struct ipmi_rs {
 
 /* These values are IANA numbers */
 typedef enum IPMI_OEM {
-    IPMI_OEM_UNKNOWN    = 0,
-    IPMI_OEM_SUN        = 42,
-    IPMI_OEM_NOKIA      = 94,
-    IPMI_OEM_INTEL      = 343,
-    IPMI_OEM_TYAN       = 6653,
-    IPMI_OEM_NEWISYS    = 9237,
-    IPMI_OEM_SUPERMICRO = 10876,
-    IPMI_OEM_GOOGLE	   = 11129,
-    IPMI_OEM_PICMG      = 12634,
-    IPMI_OEM_KONTRON    = 15000,
+     IPMI_OEM_UNKNOWN    = 0,
+     IPMI_OEM_HP         = 11,
+     IPMI_OEM_SUN        = 42,
+     IPMI_OEM_NOKIA      = 94,
+     IPMI_OEM_BULL       = 107,
+     IPMI_OEM_HITACHI_116 = 116,
+     IPMI_OEM_NEC        = 119,
+     IPMI_OEM_TOSHIBA    = 186,
+     IPMI_OEM_INTEL      = 343,
+     IPMI_OEM_TATUNG     = 373,
+     IPMI_OEM_HITACHI_399 = 399,
+     IPMI_OEM_DELL       = 674,
+     IPMI_OEM_LMC        = 2168,
+     IPMI_OEM_RADISYS    = 4337,
+     IPMI_OEM_MAGNUM     = 5593,
+     IPMI_OEM_TYAN       = 6653,
+     IPMI_OEM_NEWISYS    = 9237,
+     IPMI_OEM_FUJITSU_SIEMENS = 10368,
+     IPMI_OEM_AVOCENT    = 10418,
+     IPMI_OEM_PEPPERCON  = 10437,
+     IPMI_OEM_SUPERMICRO = 10876,
+     IPMI_OEM_OSA        = 11102,
+     IPMI_OEM_GOOGLE     = 11129,
+     IPMI_OEM_PICMG      = 12634,
+     IPMI_OEM_RARITAN    = 13742,
+     IPMI_OEM_KONTRON    = 15000,
+     IPMI_OEM_PPS        = 16394,
+     IPMI_OEM_AMI        = 20974,
+     IPMI_OEM_NOKIA_SIEMENS_NETWORKS = 28458
 } IPMI_OEM;
 
 extern const struct valstr completion_code_vals[];
