@@ -165,6 +165,7 @@ struct ipmi_intf {
 	int opened;
 	int abort;
 	int noanswer;
+	IPMI_OEM manufacturer_id;
 
 	struct ipmi_session * session;
 	struct ipmi_oem_handle * oem;
@@ -175,6 +176,7 @@ struct ipmi_intf {
 	uint8_t target_channel;
 	uint32_t transit_addr;
 	uint8_t transit_channel;
+	uint8_t channel_buf_size;
 
 	uint8_t devnum;
 
