@@ -105,6 +105,9 @@ enum {
 #define GET_SENSOR_READING      0x2d
 #define GET_SENSOR_TYPE         0x2f
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_repo_info_rs {
 	uint8_t version;	/* SDR version (51h) */
 	uint16_t count;		/* number of records */
@@ -112,20 +115,39 @@ struct sdr_repo_info_rs {
 	uint32_t add_stamp;	/* last add timestamp */
 	uint32_t erase_stamp;	/* last del timestamp */
 	uint8_t op_support;	/* supported operations */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 /* builtin (device) sdrs support */
 struct sdr_device_info_rs {
 	unsigned char count;	/* number of records */
 	unsigned char flags;	/* flags */
 	unsigned char popChangeInd[3];	/* free space in SDR */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 #define GET_SDR_RESERVE_REPO	0x22
 struct sdr_reserve_repo_rs {
 	uint16_t reserve_id;	/* reservation ID */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 #define GET_SDR		0x23
 struct sdr_get_rq {
 	uint16_t reserve_id;	/* reservation ID */
@@ -133,8 +155,14 @@ struct sdr_get_rq {
 	uint8_t offset;		/* offset into SDR */
 #define GET_SDR_ENTIRE_RECORD	0xff
 	uint8_t length;		/* length to read */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_get_rs {
 	uint16_t next;		/* next record id */
 	uint16_t id;		/* record ID */
@@ -152,15 +180,21 @@ struct sdr_get_rs {
 #define SDR_RECORD_TYPE_OEM			0xc0
 	uint8_t type;		/* record type */
 	uint8_t length;		/* remaining record bytes */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_mask {
 	union {
 		struct {
 			uint16_t assert_event;	/* assertion event mask */
 			uint16_t deassert_event;	/* de-assertion event mask */
 			uint16_t read;	/* discrete reading mask */
-		} discrete;
+		} ATTRIBUTE_PACKING discrete;
 		struct {
 #if WORDS_BIGENDIAN
 			uint16_t reserved:1;
@@ -254,7 +288,7 @@ struct sdr_record_mask {
 					uint16_t unr:1;
 					uint16_t reserved:2;
 #endif
-				} set;
+				} ATTRIBUTE_PACKING set;
 				struct {
 #if WORDS_BIGENDIAN			/* readable threshold mask */
 					/* padding upper 8 bits */
@@ -276,13 +310,19 @@ struct sdr_record_mask {
 					uint16_t reserved:2;
 					uint16_t settable:8;
 #endif
-				} read;
-			};
-		} threshold;
-	} type;
-} __attribute__ ((packed));
+				} ATTRIBUTE_PACKING read;
+			} ATTRIBUTE_PACKING;
+		} ATTRIBUTE_PACKING threshold;
+	} ATTRIBUTE_PACKING type;
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
-struct sdr_record_compact_sensor {
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
+struct sdr_record_common_sensor {
 	struct {
 		uint8_t owner_id;
 #if WORDS_BIGENDIAN
@@ -295,7 +335,7 @@ struct sdr_record_compact_sensor {
 		uint8_t channel:4;	/* channel number */
 #endif
 		uint8_t sensor_num;	/* unique sensor number */
-	} keys;
+	} ATTRIBUTE_PACKING keys;
 
 	struct entity_id entity;
 
@@ -320,7 +360,7 @@ struct sdr_record_compact_sensor {
 			uint8_t scanning:1;
 			uint8_t __reserved:1;
 #endif
-		} init;
+		} ATTRIBUTE_PACKING init;
 		struct {
 #if WORDS_BIGENDIAN
 			uint8_t ignore:1;
@@ -335,9 +375,9 @@ struct sdr_record_compact_sensor {
 			uint8_t rearm:1;
 			uint8_t ignore:1;
 #endif
-		} capabilities;
-		uint8_t type;	/* sensor type */
-	} sensor;
+		} ATTRIBUTE_PACKING capabilities;
+		uint8_t type;
+	} ATTRIBUTE_PACKING sensor;
 
 	uint8_t event_type;	/* event/reading type code */
 
@@ -358,9 +398,23 @@ struct sdr_record_compact_sensor {
 		struct {
 			uint8_t base;
 			uint8_t modifier;
-		} type;
-	} unit;
+		} ATTRIBUTE_PACKING type;
+	} ATTRIBUTE_PACKING unit;
+} ATTRIBUTE_PACKING;
 
+/* SDR Record Common Sensor header macros */
+#define IS_THRESHOLD_SENSOR(s)	((s)->event_type == 1)
+#define UNITS_ARE_DISCRETE(s)	((s)->unit.analog == 3)
+
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
+
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
+struct sdr_record_compact_sensor {
+	struct sdr_record_common_sensor cmn;
 	struct {
 #if WORDS_BIGENDIAN
 		uint8_t __reserved:2;
@@ -378,21 +432,27 @@ struct sdr_record_compact_sensor {
 		uint8_t mod_offset:7;
 		uint8_t entity_inst:1;
 #endif
-	} share;
+	} ATTRIBUTE_PACKING share;
 
 	struct {
 		struct {
 			uint8_t positive;
 			uint8_t negative;
-		} hysteresis;
-	} threshold;
+		} ATTRIBUTE_PACKING hysteresis;
+	} ATTRIBUTE_PACKING threshold;
 
 	uint8_t __reserved[3];
 	uint8_t oem;		/* reserved for OEM use */
 	uint8_t id_code;	/* sensor ID string type/length code */
 	uint8_t id_string[16];	/* sensor ID string bytes, only if id_code != 0 */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_eventonly_sensor {
 	struct {
 		uint8_t owner_id;
@@ -406,7 +466,7 @@ struct sdr_record_eventonly_sensor {
 		uint8_t channel:4;	/* channel number */
 #endif
 		uint8_t sensor_num;	/* unique sensor number */
-	} keys;
+	} ATTRIBUTE_PACKING keys;
 
 	struct entity_id entity;
 
@@ -430,93 +490,23 @@ struct sdr_record_eventonly_sensor {
 		uint8_t mod_offset:7;
 		uint8_t entity_inst:1;
 #endif
-	} share;
+	} ATTRIBUTE_PACKING share;
 
 	uint8_t __reserved;
 	uint8_t oem;		/* reserved for OEM use */
 	uint8_t id_code;	/* sensor ID string type/length code */
 	uint8_t id_string[16];	/* sensor ID string bytes, only if id_code != 0 */
 
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_full_sensor {
-	struct {
-		uint8_t owner_id;
-#if WORDS_BIGENDIAN
-		uint8_t channel:4;	/* channel number */
-		uint8_t __reserved:2;
-		uint8_t lun:2;	/* sensor owner lun */
-#else
-		uint8_t lun:2;	/* sensor owner lun */
-		uint8_t __reserved:2;
-		uint8_t channel:4;	/* channel number */
-#endif
-		uint8_t sensor_num;	/* unique sensor number */
-	} keys;
-
-	struct entity_id entity;
-
-	struct {
-		struct {
-#if WORDS_BIGENDIAN
-			uint8_t __reserved:1;
-			uint8_t scanning:1;
-			uint8_t events:1;
-			uint8_t thresholds:1;
-			uint8_t hysteresis:1;
-			uint8_t type:1;
-			uint8_t event_gen:1;
-			uint8_t sensor_scan:1;
-#else
-			uint8_t sensor_scan:1;
-			uint8_t event_gen:1;
-			uint8_t type:1;
-			uint8_t hysteresis:1;
-			uint8_t thresholds:1;
-			uint8_t events:1;
-			uint8_t scanning:1;
-			uint8_t __reserved:1;
-#endif
-		} init;
-		struct {
-#if WORDS_BIGENDIAN
-			uint8_t ignore:1;
-			uint8_t rearm:1;
-			uint8_t hysteresis:2;
-			uint8_t threshold:2;
-			uint8_t event_msg:2;
-#else
-			uint8_t event_msg:2;
-			uint8_t threshold:2;
-			uint8_t hysteresis:2;
-			uint8_t rearm:1;
-			uint8_t ignore:1;
-#endif
-		} capabilities;
-		uint8_t type;
-	} sensor;
-
-	uint8_t event_type;	/* event/reading type code */
-
-	struct sdr_record_mask mask;
-
-	struct {
-#if WORDS_BIGENDIAN
-		uint8_t analog:2;
-		uint8_t rate:3;
-		uint8_t modifier:2;
-		uint8_t pct:1;
-#else
-		uint8_t pct:1;
-		uint8_t modifier:2;
-		uint8_t rate:3;
-		uint8_t analog:2;
-#endif
-		struct {
-			uint8_t base;
-			uint8_t modifier;
-		} type;
-	} unit;
+	struct sdr_record_common_sensor cmn;
 
 #define SDR_SENSOR_L_LINEAR     0x00
 #define SDR_SENSOR_L_LN         0x01
@@ -548,7 +538,7 @@ struct sdr_record_full_sensor {
 		uint8_t normal_min:1;	/* normal min field specified */
 		uint8_t __reserved:5;
 #endif
-	} analog_flag;
+	} ATTRIBUTE_PACKING analog_flag;
 
 	uint8_t nominal_read;	/* nominal reading, raw value */
 	uint8_t normal_max;	/* normal maximum, raw value */
@@ -561,23 +551,29 @@ struct sdr_record_full_sensor {
 			uint8_t non_recover;
 			uint8_t critical;
 			uint8_t non_critical;
-		} upper;
+		} ATTRIBUTE_PACKING upper;
 		struct {
 			uint8_t non_recover;
 			uint8_t critical;
 			uint8_t non_critical;
-		} lower;
+		} ATTRIBUTE_PACKING lower;
 		struct {
 			uint8_t positive;
 			uint8_t negative;
-		} hysteresis;
-	} threshold;
+		} ATTRIBUTE_PACKING hysteresis;
+	} ATTRIBUTE_PACKING threshold;
 	uint8_t __reserved[2];
 	uint8_t oem;		/* reserved for OEM use */
 	uint8_t id_code;	/* sensor ID string type/length code */
 	uint8_t id_string[16];	/* sensor ID string bytes, only if id_code != 0 */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_mc_locator {
 	uint8_t dev_slave_addr;
 #if WORDS_BIGENDIAN
@@ -602,7 +598,10 @@ struct sdr_record_mc_locator {
 	uint8_t oem;
 	uint8_t id_code;
 	uint8_t id_string[16];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
 struct sdr_record_fru_locator {
 	uint8_t dev_slave_addr;
@@ -632,8 +631,14 @@ struct sdr_record_fru_locator {
 	uint8_t oem;
 	uint8_t id_code;
 	uint8_t id_string[16];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_generic_locator {
 	uint8_t dev_access_addr;
 	uint8_t dev_slave_addr;
@@ -660,8 +665,14 @@ struct sdr_record_generic_locator {
 	uint8_t oem;
 	uint8_t id_code;
 	uint8_t id_string[16];
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_entity_assoc {
 	struct entity_id entity;	/* container entity ID and instance */
 	struct {
@@ -685,7 +696,10 @@ struct sdr_record_entity_assoc {
 	uint8_t entity_inst_3;	/* entity inst 3  |  range 2 first instance */
 	uint8_t entity_id_4;	/* entity ID 4    |  range 2 entity */
 	uint8_t entity_inst_4;	/* entity inst 4  |  range 2 last instance */
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
 struct sdr_record_oem {
 	uint8_t *data;
@@ -696,6 +710,9 @@ struct sdr_record_oem {
  * The Get SDR Repository Info response structure
  * From table 33-3 of the IPMI v2.0 spec
  */
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct get_sdr_repository_info_rsp {
 	uint8_t sdr_version;
 	uint8_t record_count_lsb;
@@ -720,7 +737,10 @@ struct get_sdr_repository_info_rsp {
 	uint8_t modal_update_support:2;
 	uint8_t overflow_flag:1;
 #endif
-} __attribute__ ((packed));
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
 
 struct ipmi_sdr_iterator {
 	uint16_t reservation;
@@ -729,6 +749,9 @@ struct ipmi_sdr_iterator {
 	int use_built_in;
 };
 
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(1)
+#endif
 struct sdr_record_list {
 	uint16_t id;
 	uint8_t version;
@@ -737,6 +760,7 @@ struct sdr_record_list {
 	uint8_t *raw;
 	struct sdr_record_list *next;
 	union {
+		struct sdr_record_common_sensor *common;
 		struct sdr_record_full_sensor *full;
 		struct sdr_record_compact_sensor *compact;
 		struct sdr_record_eventonly_sensor *eventonly;
@@ -745,8 +769,12 @@ struct sdr_record_list {
 		struct sdr_record_mc_locator *mcloc;
 		struct sdr_record_entity_assoc *entassoc;
 		struct sdr_record_oem *oem;
-	} record;
-};
+	} ATTRIBUTE_PACKING  record;
+} ATTRIBUTE_PACKING;
+#ifdef HAVE_PRAGMA_PACK
+#pragma pack(0)
+#endif
+
 
 /* unit description codes (IPMI v1.5 section 37.16) */
 #define UNIT_MAX	0x90
@@ -792,6 +820,22 @@ static const char *sensor_type_desc[] __attribute__ ((unused)) = {
 	    "Management Subsystem Health", "Battery","Session Audit",
        "Version Change","FRU State" };
 
+struct sensor_reading {
+	char		s_id[17];		/* name of the sensor */
+	struct sdr_record_full_sensor    *full;
+	struct sdr_record_compact_sensor *compact;
+	uint8_t		s_reading_valid;	/* read value valididity */
+	uint8_t		s_scanning_disabled;	/* read of value disabled */
+	uint8_t		s_reading_unavailable;	/* read value unavailable */
+	uint8_t		s_reading;		/* value which was read */
+	uint8_t		s_data2;		/* data2 value read */
+	uint8_t		s_data3;		/* data3 value read */
+	uint8_t		s_has_analog_value;	/* sensor has analog value */
+	double		s_a_val;		/* read value converted to analog */
+	char		s_a_str[16];		/* analog value as a string */
+	const char	*s_a_units;		/* analog value units string */
+};
+
 struct ipmi_sdr_iterator *ipmi_sdr_start(struct ipmi_intf *intf,
                                          int use_builtin);
 struct sdr_get_rs *ipmi_sdr_get_next_header(struct ipmi_intf *intf,
@@ -800,13 +844,26 @@ uint8_t *ipmi_sdr_get_record(struct ipmi_intf *intf, struct sdr_get_rs *header,
 			     struct ipmi_sdr_iterator *i);
 void ipmi_sdr_end(struct ipmi_intf *intf, struct ipmi_sdr_iterator *i);
 int ipmi_sdr_print_sdr(struct ipmi_intf *intf, uint8_t type);
+
+int ipmi_sdr_print_name_from_rawentry(struct ipmi_intf *intf,uint16_t id,
+                                      uint8_t type,uint8_t * raw);
 int ipmi_sdr_print_rawentry(struct ipmi_intf *intf, uint8_t type, uint8_t * raw,
 			    int len);
 int ipmi_sdr_print_listentry(struct ipmi_intf *intf,
 			     struct sdr_record_list *entry);
-char *ipmi_sdr_get_unit_string(uint8_t type, uint8_t base, uint8_t modifier);
-const char *ipmi_sdr_get_status(struct sdr_record_full_sensor *sensor,
-				uint8_t stat);
+void ipmi_sdr_print_sensor_hysteresis(struct sdr_record_common_sensor *sensor,
+		 struct sdr_record_full_sensor   *full,
+		 uint8_t hysteresis_value,
+		 const char *hdrstr);
+const char *ipmi_sdr_get_unit_string(uint8_t pct, uint8_t type,
+				      uint8_t base, uint8_t modifier);
+struct sensor_reading *
+ipmi_sdr_read_sensor_value(struct ipmi_intf *intf,
+		struct sdr_record_common_sensor *sensor,
+		uint8_t sdr_record_type, int precision);
+const char *ipmi_sdr_get_thresh_status(struct sensor_reading *sr,
+					const char *invalidstr);
+const char *ipmi_sdr_get_status(int, const char *, uint8_t stat);
 double sdr_convert_sensor_tolerance(struct sdr_record_full_sensor *sensor,
 				  uint8_t val);
 double sdr_convert_sensor_reading(struct sdr_record_full_sensor *sensor,
@@ -820,21 +877,18 @@ struct ipmi_rs *ipmi_sdr_get_sensor_reading(struct ipmi_intf *intf,
 struct ipmi_rs *ipmi_sdr_get_sensor_reading_ipmb(struct ipmi_intf *intf,
 						 uint8_t sensor,
 						 uint8_t target,
-						 uint8_t lun);
+						 uint8_t lun,
+						 uint8_t channel);
 struct ipmi_rs *ipmi_sdr_get_sensor_thresholds(struct ipmi_intf *intf,
 					       uint8_t sensor,
-					       uint8_t target, uint8_t lun);
+					       uint8_t target, uint8_t lun, uint8_t channel);
 struct ipmi_rs *ipmi_sdr_get_sensor_hysteresis(struct ipmi_intf *intf,
 					       uint8_t sensor,
-					       uint8_t target, uint8_t lun);
+					       uint8_t target, uint8_t lun, uint8_t channel);
 const char *ipmi_sdr_get_sensor_type_desc(const uint8_t type);
 int ipmi_sdr_get_reservation(struct ipmi_intf *intf, int use_builtin,
                              uint16_t * reserve_id);
 
-int ipmi_sdr_print_sensor_full(struct ipmi_intf *intf,
-			       struct sdr_record_full_sensor *sensor);
-int ipmi_sdr_print_sensor_compact(struct ipmi_intf *intf,
-				  struct sdr_record_compact_sensor *sensor);
 int ipmi_sdr_print_sensor_eventonly(struct ipmi_intf *intf,
 				    struct sdr_record_eventonly_sensor *sensor);
 int ipmi_sdr_print_sensor_generic_locator(struct ipmi_intf *intf,
@@ -864,16 +918,16 @@ int ipmi_sdr_print_info(struct ipmi_intf *intf);
 void ipmi_sdr_print_discrete_state(const char *desc, uint8_t sensor_type,
 				   uint8_t event_type, uint8_t state1,
 				   uint8_t state2);
-void ipmi_sdr_print_discrete_state_mini(const char *separator,
+void ipmi_sdr_print_discrete_state_mini(const char *header, const char *separator,
 					uint8_t sensor_type, uint8_t event_type,
 					uint8_t state1, uint8_t state2);
 int ipmi_sdr_print_sensor_event_status(struct ipmi_intf *intf,
 				       uint8_t sensor_num, uint8_t sensor_type,
 				       uint8_t event_type, int numeric_fmt,
-				       uint8_t target, uint8_t lun);
+				       uint8_t target, uint8_t lun, uint8_t channel);
 int ipmi_sdr_print_sensor_event_enable(struct ipmi_intf *intf,
 				       uint8_t sensor_num, uint8_t sensor_type,
 				       uint8_t event_type, int numeric_fmt,
-				       uint8_t target, uint8_t lun);
+				       uint8_t target, uint8_t lun, uint8_t channel);
 
 #endif				/* IPMI_SDR_H */
