@@ -35,6 +35,7 @@
 #include <sys/ioctl.h>  /* For TIOCNOTTY */
 
 #include <stdlib.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <inttypes.h>
 #include <signal.h>
@@ -73,13 +74,13 @@ uint16_t buf2short(uint8_t * buf)
 
 const char * buf2str(uint8_t * buf, int len)
 {
-	static char str[1024];
+	static char str[2049];
 	int i;
 
 	if (len <= 0 || len > 1024)
 		return NULL;
 
-	memset(str, 0, 1024);
+	memset(str, 0, 2049);
 
 	for (i=0; i<len; i++)
 		sprintf(str+i+i, "%2.2x", buf[i]);
@@ -99,13 +100,13 @@ void printbuf(const uint8_t * buf, int len, const char * desc)
 	if (verbose < 1)
 		return;
 
-	fprintf(stderr, "%s (%d bytes)\r\n", desc, len);
+	fprintf(stderr, "%s (%d bytes)\n", desc, len);
 	for (i=0; i<len; i++) {
 		if (((i%16) == 0) && (i != 0))
-			fprintf(stderr, "\r\n");
+			fprintf(stderr, "\n");
 		fprintf(stderr, " %2.2x", buf[i]);
 	}
-	fprintf(stderr, "\r\n");
+	fprintf(stderr, "\n");
 }
 
 const char * val2str(uint16_t val, const struct valstr *vs)
@@ -123,28 +124,19 @@ const char * val2str(uint16_t val, const struct valstr *vs)
 
 	return un_str;
 }
- 
+
 const char * oemval2str(uint32_t oem, uint16_t val,
                                              const struct oemvalstr *vs)
 {
 	static char un_str[32];
 	int i;
 
-	for (i = 0; vs[i].oem != 0x00 &&  vs[i].str != NULL; i++) {
-      /* FIXME: for now on we assume PICMG capability on all IANAs */
-      if
-      ( 
-         (
-            vs[i].oem == oem 
-            ||
-            vs[i].oem == IPMI_OEM_PICMG 
-         )
-         &&  
-         vs[i].val == val  
-      )
-      {
-         return vs[i].str;
-      }
+	for (i = 0; vs[i].oem != 0xffffff &&  vs[i].str != NULL; i++) {
+		/* FIXME: for now on we assume PICMG capability on all IANAs */
+		if ( (vs[i].oem == oem || vs[i].oem == IPMI_OEM_PICMG) &&
+				vs[i].val == val ) {
+			return vs[i].str;
+		}
 	}
 
 	memset(un_str, 0, 32);
@@ -153,6 +145,180 @@ const char * oemval2str(uint32_t oem, uint16_t val,
 	return un_str;
 }
 
+/* str2long - safely convert string to int64_t
+ *
+ * @str: source string to convert from
+ * @lng_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) invalid input, (-3) for *flow
+ */
+int str2long(const char * str, int64_t * lng_ptr)
+{
+	char * end_ptr = 0;
+	if (!str || !lng_ptr)
+		return (-1);
+
+	*lng_ptr = 0;
+	errno = 0;
+	*lng_ptr = strtol(str, &end_ptr, 0);
+
+	if (*end_ptr != '\0')
+		return (-2);
+
+	if (errno != 0)
+		return (-3);
+	
+	return 0;
+} /* str2long(...) */
+
+/* str2ulong - safely convert string to uint64_t
+ *
+ * @str: source string to convert from
+ * @ulng_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) invalid input, (-3) for *flow
+ */
+int str2ulong(const char * str, uint64_t * ulng_ptr)
+{
+	char * end_ptr = 0;
+	if (!str || !ulng_ptr)
+		return (-1);
+
+	*ulng_ptr = 0;
+	errno = 0;
+	*ulng_ptr = strtoul(str, &end_ptr, 0);
+
+	if (*end_ptr != '\0')
+		return (-2);
+
+	if (errno != 0)
+		return (-3);
+
+	return 0;
+} /* str2ulong(...) */
+
+/* str2int - safely convert string to int32_t
+ *
+ * @str: source string to convert from
+ * @int_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) invalid input, (-3) for *flow
+ */
+int str2int(const char * str, int32_t * int_ptr)
+{
+	int rc = 0;
+	int64_t arg_long = 0;
+	if ( (rc = str2long(str, &arg_long)) != 0 ) {
+		*int_ptr = 0;
+		return rc;
+	}
+
+	if (arg_long < INT32_MIN || arg_long > INT32_MAX)
+		return (-3);
+
+	*int_ptr = (int32_t)arg_long;
+	return 0;
+} /* str2int(...) */
+
+/* str2uint - safely convert string to uint32_t
+ *
+ * @str: source string to convert from
+ * @uint_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) invalid input, (-3) for *flow
+ */
+int str2uint(const char * str, uint32_t * uint_ptr)
+{
+	int rc = 0;
+	uint64_t arg_ulong = 0;
+	if ( (rc = str2ulong(str, &arg_ulong)) != 0) {
+		*uint_ptr = 0;
+		return rc;
+	}
+
+	if (arg_ulong > UINT32_MAX)
+		return (-3);
+
+	*uint_ptr = (uint32_t)arg_ulong;
+	return 0;
+} /* str2uint(...) */
+
+/* str2short - safely convert string to int16_t
+ *
+ * @str: source string to convert from
+ * @shrt_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) invalid input, (-3) for *flow
+ */
+int str2short(const char * str, int16_t * shrt_ptr)
+{
+	int rc = (-3);
+	int64_t arg_long = 0;
+	if ( (rc = str2long(str, &arg_long)) != 0 ) {
+		*shrt_ptr = 0;
+		return rc;
+	}
+
+	if (arg_long < INT16_MIN || arg_long > INT16_MAX)
+		return (-3);
+
+	*shrt_ptr = (int16_t)arg_long;
+	return 0;
+} /* str2short(...) */
+
+/* str2ushort - safely convert string to uint16_t
+ *
+ * @str: source string to convert from
+ * @ushrt_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) invalid input, (-3) for *flow
+ */
+int str2ushort(const char * str, uint16_t * ushrt_ptr)
+{
+	int rc = (-3);
+	uint64_t arg_ulong = 0;
+	if ( (rc = str2ulong(str, &arg_ulong)) != 0 ) {
+		*ushrt_ptr = 0;
+		return rc;
+	}
+
+	if (arg_ulong > UINT16_MAX)
+		return (-3);
+
+	*ushrt_ptr = (uint16_t)arg_ulong;
+	return 0;
+} /* str2ushort(...) */
+
+/* str2uchar - safely convert string to uint8
+ *
+ * @str: source string to convert from
+ * @uchr_ptr: pointer where to store result
+ *
+ * returns zero on success
+ * returns (-1) if one of args is NULL, (-2) or (-3) if conversion fails
+ */
+int str2uchar(const char * str, uint8_t * uchr_ptr)
+{
+	int rc = (-3);
+	uint64_t arg_ulong = 0;
+	if ( (rc = str2ulong(str, &arg_ulong)) != 0 ) {
+		*uchr_ptr = 0;
+		return rc;
+	}
+
+	if (arg_ulong > UINT8_MAX)
+		return (-3);
+
+	*uchr_ptr = (uint8_t)arg_ulong;
+	return 0;
+} /* str2uchar(...) */
+
 uint16_t str2val(const char *str, const struct valstr *vs)
 {
 	int i;
