@@ -123,6 +123,7 @@ ipmi_chassis_identify(struct ipmi_intf * intf, char * arg)
 {
    struct ipmi_rq req;
    struct ipmi_rs * rsp;
+   int rc = (-3);
 
    struct {
       uint8_t interval;
@@ -138,8 +139,16 @@ ipmi_chassis_identify(struct ipmi_intf * intf, char * arg)
          identify_data.interval = 0;
          identify_data.force_on = 1;
       } else {
-         identify_data.interval = (uint8_t)atoi(arg);
+         identify_data.interval = 0;
          identify_data.force_on = 0;
+         if ( (rc = str2uchar(arg, &identify_data.interval)) != 0) {
+            if (rc == (-2)) {
+                lprintf(LOG_ERR, "Invalid interval given.");
+            } else {
+                lprintf(LOG_ERR, "Given interval is too big.");
+            }
+            return (-1);
+         }
       }
       req.msg.data = (uint8_t *)&identify_data;
       /* The Force Identify On byte is optional and not
@@ -222,10 +231,10 @@ ipmi_chassis_poh(struct ipmi_intf * intf)
    minutes -= hours * 60;
 
    if (mins_per_count < 60) {
-       printf("POH Counter  : %li days, %li hours, %li minutes\n",
+       printf("POH Counter  : %i days, %i hours, %li minutes\n",
                days, hours, (long)minutes);
    } else {
-       printf("POH Counter  : %li days, %li hours\n", days, hours);
+       printf("POH Counter  : %i days, %i hours\n", days, hours);
    }
 
    return 0;
@@ -483,14 +492,20 @@ ipmi_chassis_get_bootparam(struct ipmi_intf * intf, char * arg)
    struct ipmi_rs * rsp;
    struct ipmi_rq req;
    uint8_t msg_data[3];
-   unsigned char param_id;
+   uint8_t param_id = 0;
 
    if (arg == NULL)
       return -1;
 
+   if (str2uchar(arg, &param_id) != 0) {
+      lprintf(LOG_ERR, "Invalid parameter '%s' given instead of bootparam.",
+           arg);
+      return (-1);
+   }
+
    memset(msg_data, 0, 3);
 
-   msg_data[0] = (uint8_t)atoi(arg) & 0x7f;
+   msg_data[0] = param_id & 0x7f;
    msg_data[1] = 0;
    msg_data[2] = 0;
 
@@ -514,6 +529,7 @@ ipmi_chassis_get_bootparam(struct ipmi_intf * intf, char * arg)
    if (verbose > 2)
       printbuf(rsp->data, rsp->data_len, "Boot Option");
 
+   param_id = 0;
    param_id = (rsp->data[1] & 0x7f);
 
    printf("Boot parameter version: %d\n", rsp->data[0]);
@@ -714,14 +730,14 @@ ipmi_chassis_get_bootparam(struct ipmi_intf * intf, char * arg)
 
          printf(" Boot Initiator Info :\n");
          printf("    Channel Number : %d\n", (rsp->data[2] & 0x0f));
-         printf("    Session Id     : %08Xh\n",session_id);
+         printf("    Session Id     : %08lXh\n",session_id);
          if(timestamp != 0)
          {
-            printf("    Timestamp      : %08Xh, %s\n",timestamp,time_buf);
+            printf("    Timestamp      : %08lXh, %s\n",timestamp,time_buf);
          }
          else
          {
-            printf("    Timestamp      : %08Xh, undefined\n",timestamp);
+            printf("    Timestamp      : %08lXh, undefined\n",timestamp);
          }
 
       }
@@ -783,30 +799,30 @@ ipmi_chassis_set_bootdev(struct ipmi_intf * intf, char * arg, uint8_t *iflags)
          memcpy(flags, iflags, sizeof (flags));
 
    if (arg == NULL)
-      flags[1] = 0x00;
+      flags[1] |= 0x00;
    else if (strncmp(arg, "none", 4) == 0)
-      flags[1] = 0x00;
+      flags[1] |= 0x00;
    else if (strncmp(arg, "pxe", 3) == 0 ||
        strncmp(arg, "force_pxe", 9) == 0)
-      flags[1] = 0x04;
+      flags[1] |= 0x04;
    else if (strncmp(arg, "disk", 4) == 0 ||
        strncmp(arg, "force_disk", 10) == 0)
-      flags[1] = 0x08;
+      flags[1] |= 0x08;
    else if (strncmp(arg, "safe", 4) == 0 ||
        strncmp(arg, "force_safe", 10) == 0)
-      flags[1] = 0x0c;
+      flags[1] |= 0x0c;
    else if (strncmp(arg, "diag", 4) == 0 ||
        strncmp(arg, "force_diag", 10) == 0)
-      flags[1] = 0x10;
+      flags[1] |= 0x10;
    else if (strncmp(arg, "cdrom", 5) == 0 ||
        strncmp(arg, "force_cdrom", 11) == 0)
-      flags[1] = 0x14;
+      flags[1] |= 0x14;
    else if (strncmp(arg, "floppy", 6) == 0 ||
        strncmp(arg, "force_floppy", 12) == 0)
-      flags[1] = 0x3c;
+      flags[1] |= 0x3c;
    else if (strncmp(arg, "bios", 4) == 0 ||
        strncmp(arg, "force_bios", 10) == 0)
-      flags[1] = 0x18;
+      flags[1] |= 0x18;
    else {
       lprintf(LOG_ERR, "Invalid argument: %s", arg);
       if (use_progress) {
@@ -1140,8 +1156,6 @@ ipmi_chassis_main(struct ipmi_intf * intf, int argc, char ** argv)
             memset(&flags[0], 0, sizeof(flags));
             token = strtok_r(argv[2] + 8, ",", &saveptr);
             while (token != NULL) {
-               int i;
-
                if (strcmp(token, "help") == 0) {
                   optionError = 1;
                   break;
