@@ -97,7 +97,7 @@ ipmi_send_platform_event(struct ipmi_intf * intf, struct platform_event_msg * em
 	if (chmed == IPMI_CHANNEL_MEDIUM_SYSTEM) {
 		/* system interface, need extra generator ID */
 		req.msg.data_len = 8;
-		rqdata[0] = 0x20;
+		rqdata[0] = 0x41;   // As per Fig. 29-2 and Table 5-4
 		memcpy(rqdata+1, emsg, sizeof(struct platform_event_msg));
 	}
 	else {
@@ -227,8 +227,8 @@ ipmi_event_find_offset(uint8_t code,
 static void
 print_sensor_states(uint8_t sensor_type, uint8_t event_type)
 {
-	printf("Sensor States: \n  ");
-	ipmi_sdr_print_discrete_state_mini("\n  ", sensor_type,
+	ipmi_sdr_print_discrete_state_mini(
+			"Sensor States: \n  ", "\n  ", sensor_type,
 					   event_type, 0xff, 0xff);
 	printf("\n");
 }
@@ -241,7 +241,7 @@ ipmi_event_fromsensor(struct ipmi_intf * intf, char * id, char * state, char * e
 	struct sdr_record_list * sdr;
 	struct platform_event_msg emsg;
 	int off;
-	uint8_t target, lun;
+	uint8_t target, lun, channel;
 
 	if (id == NULL) {
 		lprintf(LOG_ERR, "No sensor ID supplied");
@@ -273,23 +273,15 @@ ipmi_event_fromsensor(struct ipmi_intf * intf, char * id, char * state, char * e
 	switch (sdr->type)
 	{
 	case SDR_RECORD_TYPE_FULL_SENSOR:
-
-		emsg.sensor_type   = sdr->record.full->sensor.type;
-		emsg.sensor_num    = sdr->record.full->keys.sensor_num;
-		emsg.event_type    = sdr->record.full->event_type;
-		target    = sdr->record.full->keys.owner_id;
-		lun    = sdr->record.full->keys.lun;
-		break;
-
 	case SDR_RECORD_TYPE_COMPACT_SENSOR:
 
-		emsg.sensor_type = sdr->record.compact->sensor.type;
-		emsg.sensor_num  = sdr->record.compact->keys.sensor_num;
-		emsg.event_type  = sdr->record.compact->event_type;
-		target    = sdr->record.compact->keys.owner_id;
-		lun    = sdr->record.compact->keys.lun;
+		emsg.sensor_type   = sdr->record.common->sensor.type;
+		emsg.sensor_num    = sdr->record.common->keys.sensor_num;
+		emsg.event_type    = sdr->record.common->event_type;
+		target    = sdr->record.common->keys.owner_id;
+		lun    = sdr->record.common->keys.lun;
+		channel = sdr->record.common->keys.channel;
 		break;
-
 	default:
 		lprintf(LOG_ERR, "Unknown sensor type for id '%s'", id);
 		return -1;
@@ -353,7 +345,7 @@ ipmi_event_fromsensor(struct ipmi_intf * intf, char * id, char * state, char * e
 		}
 
 		rsp = ipmi_sdr_get_sensor_thresholds(intf, emsg.sensor_num,
-							target, lun);
+							target, lun, channel);
 
 		if (rsp != NULL && rsp->ccode == 0) {
 
@@ -361,7 +353,7 @@ ipmi_event_fromsensor(struct ipmi_intf * intf, char * id, char * state, char * e
 			emsg.event_data[2] = rsp->data[(emsg.event_data[0] / 2) + 1];
 
 			rsp = ipmi_sdr_get_sensor_hysteresis(intf, emsg.sensor_num,
-								target, lun);
+								target, lun, channel);
 			if (rsp != NULL && rsp->ccode == 0)
 				off = dir ? rsp->data[0] : rsp->data[1];
 			if (off <= 0)
@@ -509,7 +501,7 @@ ipmi_event_fromfile(struct ipmi_intf * intf, char * file)
 	chmed = ipmi_current_channel_medium(intf);
 	if (chmed == IPMI_CHANNEL_MEDIUM_SYSTEM) {
 		/* system interface, need extra generator ID */
-		rqdata[0] = 0x20;
+		rqdata[0] = 0x41;   // As per Fig. 29-2 and Table 5-4
 		req.msg.data_len = 8;
 	}
 
