@@ -67,13 +67,14 @@
 #include <ipmitool/ipmi_pef.h>
 #include <ipmitool/ipmi_oem.h>
 #include <ipmitool/ipmi_ekanalyzer.h>
+#include <ipmitool/ipmi_picmg.h>
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #ifdef ENABLE_ALL_OPTIONS
-# define OPTION_STRING	"I:hVvcgsEKao:H:d:P:f:U:p:C:L:A:t:T:m:S:l:b:B:e:k:y:O:"
+# define OPTION_STRING	"I:hVvcgsEKYao:H:d:P:f:U:p:C:L:A:t:T:m:z:S:l:b:B:e:k:y:O:R:N:"
 #else
 # define OPTION_STRING	"I:hVvcH:f:U:p:d:S:"
 #endif
@@ -90,6 +91,7 @@ extern int ipmi_shell_main(struct ipmi_intf * intf, int argc, char ** argv);
 extern int ipmi_set_main(struct ipmi_intf * intf, int argc, char ** argv);
 extern int ipmi_exec_main(struct ipmi_intf * intf, int argc, char ** argv);
 
+static struct ipmi_intf * ipmi_main_intf = NULL;
 
 /* ipmi_password_file_read  -  Open file and read password from it
  *
@@ -115,19 +117,19 @@ ipmi_password_file_read(char * filename)
 	fp = ipmi_open_file_read((const char *)filename);
 	if (fp == NULL) {
 		lprintf(LOG_ERR, "Unable to open password file %s",
-			filename);
+				filename);
 		return NULL;
 	}
 
 	/* read in id */
 	if (fgets(pass, 16, fp) == NULL) {
 		lprintf(LOG_ERR, "Unable to read password from file %s",
-			filename);
+				filename);
 		fclose(fp);
 		return NULL;
 	}
 
- 	/* remove trailing whitespace */
+	/* remove trailing whitespace */
 	l = strcspn(pass, " \r\n\t");
 	if (l > 0) {
 		pass[l] = '\0';
@@ -202,6 +204,7 @@ ipmi_cmd_run(struct ipmi_intf * intf, char * name, int argc, char ** argv)
 		cmd = intf->cmdlist;
 		if (strncmp(cmd->name, "default", 7) == 0)
 			return cmd->func(intf, argc+1, argv-1);
+
 		lprintf(LOG_ERR, "Invalid command: %s", name);
 		ipmi_cmd_print(intf->cmdlist);
 		return -1;
@@ -224,9 +227,11 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -p port        Remote RMCP port [default=623]");
 	lprintf(LOG_NOTICE, "       -U username    Remote session username");
 	lprintf(LOG_NOTICE, "       -f file        Read remote session password from file");
+	lprintf(LOG_NOTICE, "       -z size        Change Size of Communication Channel (OEM)");
 	lprintf(LOG_NOTICE, "       -S sdr         Use local file for remote SDR cache");
 #ifdef ENABLE_ALL_OPTIONS
 	lprintf(LOG_NOTICE, "       -a             Prompt for remote password");
+	lprintf(LOG_NOTICE, "       -Y             Prompt for the Kg key for IPMIv2 authentication");
 	lprintf(LOG_NOTICE, "       -e char        Set SOL escape character");
 	lprintf(LOG_NOTICE, "       -C ciphersuite Cipher suite to be used by lanplus interface");
 	lprintf(LOG_NOTICE, "       -k key         Use Kg key for IPMIv2 authentication");
@@ -240,11 +245,14 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	lprintf(LOG_NOTICE, "       -m address     Set local IPMB address");
 	lprintf(LOG_NOTICE, "       -b channel     Set destination channel for bridged request");
 	lprintf(LOG_NOTICE, "       -t address     Bridge request to remote target address");
+	lprintf(LOG_NOTICE, "       -M address     Set transit local address for bridge request(dual bridge)");
 	lprintf(LOG_NOTICE, "       -B channel     Set transit channel for bridged request (dual bridge)");
 	lprintf(LOG_NOTICE, "       -T address     Set transit address for bridge request (dual bridge)");
 	lprintf(LOG_NOTICE, "       -l lun         Set destination lun for raw commands");
 	lprintf(LOG_NOTICE, "       -o oemtype     Setup for OEM (use 'list' to see available OEM types)");
 	lprintf(LOG_NOTICE, "       -O seloem      Use file for OEM SEL event descriptions");
+	lprintf(LOG_NOTICE, "       -N seconds     Specify timeout for lan [default=2] / lanplus [default=1] interface");
+	lprintf(LOG_NOTICE, "       -R retry       Set the number of retries for lan/lanplus interface [default=4]");
 #endif
 	lprintf(LOG_NOTICE, "");
 
@@ -253,6 +261,22 @@ ipmi_option_usage(const char * progname, struct ipmi_cmd * cmdlist, struct ipmi_
 	if (cmdlist != NULL)
 		ipmi_cmd_print(cmdlist);
 }
+/* ipmi_catch_sigint  -  Handle the interrupt signal (Ctrl-C), close the
+ *                       interface, and exit ipmitool with error (-1)
+ *
+ *                       This insures that the IOL session gets freed
+ *                       for other callers.
+ * 
+ * returns -1
+ */
+void ipmi_catch_sigint()
+{
+	if (ipmi_main_intf != NULL) {
+		printf("\nSIGN INT: Close Interface %s\n",ipmi_main_intf->desc);
+		ipmi_main_intf->close(ipmi_main_intf);
+	}
+	exit(-1);
+}
 
 /* ipmi_parse_hex - convert hexadecimal numbers to ascii string
  *                  Input string must be composed of two-characer hexadecimal numbers.
@@ -298,7 +322,7 @@ ipmi_parse_hex(const char *str)
 			free(out);
 			return NULL;
 		}
-		
+
 		if (*p < 'A') /* it must be 0-9 */
 			b = *p - '0';
 		else /* it's A-F or a-f */
@@ -311,7 +335,7 @@ ipmi_parse_hex(const char *str)
 			shift = 4;
 			q++;
 		}
-    }
+	}
 
 	return out;
 }
@@ -328,19 +352,23 @@ ipmi_parse_hex(const char *str)
  */
 int
 ipmi_main(int argc, char ** argv,
-	  struct ipmi_cmd * cmdlist,
-	  struct ipmi_intf_support * intflist)
+		struct ipmi_cmd * cmdlist,
+		struct ipmi_intf_support * intflist)
 {
-	struct ipmi_intf * intf = NULL;
 	struct ipmi_intf_support * sup;
 	int privlvl = 0;
 	uint8_t target_addr = 0;
 	uint8_t target_channel = 0;
+
 	uint8_t transit_addr = 0;
 	uint8_t transit_channel = 0;
 	uint8_t target_lun     = 0;
-	uint8_t my_addr = 0;
+	uint8_t my_addr = 0x20;
+	uint16_t my_long_packet_size=0;
+	uint8_t my_long_packet_set=0;
 	uint8_t lookupbit = 0x10;	/* use name-only lookup by default */
+	int retry = 0;
+	uint32_t timeout = 0;
 	int authtype = -1;
 	char * tmp = NULL;
 	char * hostname = NULL;
@@ -354,7 +382,7 @@ ipmi_main(int argc, char ** argv,
 	char * seloem   = NULL;
 	int port = 0;
 	int devnum = 0;
-	int cipher_suite_id = 3; /* See table 22-19 of the IPMIv2 spec */
+	int cipher_suite_id = 1; /* See table 22-19 of the IPMIv2 spec */
 	int argflag, i, found;
 	int rc = -1;
 	char sol_escape_char = SOL_ESCAPE_CHARACTER_DEFAULT;
@@ -362,6 +390,7 @@ ipmi_main(int argc, char ** argv,
 	/* save program name */
 	progname = strrchr(argv[0], '/');
 	progname = ((progname == NULL) ? argv[0] : progname+1);
+	signal(SIGINT, ipmi_catch_sigint);
 
 	while ((argflag = getopt(argc, (char **)argv, OPTION_STRING)) != -1)
 	{
@@ -376,8 +405,8 @@ ipmi_main(int argc, char ** argv,
 				found = 0;
 				for (sup=intflist; sup->name != NULL; sup++) {
 					if (strncmp(sup->name, intfname, strlen(intfname)) == 0 &&
-					    strncmp(sup->name, intfname, strlen(sup->name)) == 0 &&
-					    sup->supported == 1)
+							strncmp(sup->name, intfname, strlen(sup->name)) == 0 &&
+							sup->supported == 1)
 						found = 1;
 				}
 				if (!found) {
@@ -397,13 +426,45 @@ ipmi_main(int argc, char ** argv,
 			goto out_free;
 			break;
 		case 'd':
-			devnum = atoi(optarg);
+			if (str2int(optarg, &devnum) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-d'.");
+				rc = -1;
+				goto out_free;
+			}
+			/* Check if device number is -gt 0; I couldn't find limit for
+			 * kernels > 2.6, thus right side is unlimited.
+			 */
+			if (devnum < 0) {
+				lprintf(LOG_ERR, "Device number %i is out of range.", devnum);
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'p':
-			port = atoi(optarg);
+			if (str2int(optarg, &port) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-p'.");
+				rc = -1;
+				goto out_free;
+			}
+			/* Check if port is -gt 0 && port is -lt 65535 */
+			if (port < 0 || port > 65535) {
+				lprintf(LOG_ERR, "Port number %i is out of range.", port);
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'C':
-			cipher_suite_id = atoi(optarg);
+			if (str2int(optarg, &cipher_suite_id) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-C'.");
+				rc = -1;
+				goto out_free;
+			}
+			/* add check Cipher is -gt 0 */
+			if (cipher_suite_id < 0) {
+				lprintf(LOG_ERR, "Cipher suite ID %i is invalid.", cipher_suite_id);
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'v':
 			verbose++;
@@ -424,7 +485,7 @@ ipmi_main(int argc, char ** argv,
 			password = ipmi_password_file_read(optarg);
 			if (password == NULL)
 				lprintf(LOG_ERR, "Unable to read password "
-					"from file %s", optarg);
+						"from file %s", optarg);
 			break;
 		case 'a':
 #ifdef HAVE_GETPASSPHRASE
@@ -450,8 +511,7 @@ ipmi_main(int argc, char ** argv,
 			}
 			break;
 		case 'K':
-			if ((tmp = getenv("IPMI_KGKEY")))
-			{
+			if ((tmp = getenv("IPMI_KGKEY"))) {
 				if (kgkey)
 					free(kgkey);
 				kgkey = strdup(tmp);
@@ -459,8 +519,7 @@ ipmi_main(int argc, char ** argv,
 					lprintf(LOG_ERR, "%s: malloc failure", progname);
 					goto out_free;
 				}
-			}
-			else {
+			} else {
 				lprintf(LOG_WARN, "Unable to read kgkey from environment");
 			}
 			break;
@@ -470,7 +529,27 @@ ipmi_main(int argc, char ** argv,
 				goto out_free;
 			}
 			break;
+		case 'Y':
+#ifdef HAVE_GETPASSPHRASE
+			tmp = getpassphrase("Key: ");
+#else
+			tmp = getpass("Key: ");
+#endif
+			if (tmp != NULL) {
+				if (kgkey)
+					free(kgkey);
+				kgkey = strdup(tmp);
+				if (kgkey == NULL) {
+					lprintf(LOG_ERR, "%s: malloc failure", progname);
+					goto out_free;
+				}
+			}
+			break;
 		case 'U':
+			if (strlen(optarg) > 16) {
+				lprintf(LOG_ERR, "Username is too long (> 16 bytes)");
+				goto out_free;
+			}
 			username = strdup(optarg);
 			if (username == NULL) {
 				lprintf(LOG_ERR, "%s: malloc failure", progname);
@@ -492,8 +571,9 @@ ipmi_main(int argc, char ** argv,
 				goto out_free;
 			}
 			if (strncmp(oemtype, "list", 4) == 0 ||
-			    strncmp(oemtype, "help", 4) == 0) {
+					strncmp(oemtype, "help", 4) == 0) {
 				ipmi_oem_print();
+				rc = 0;
 				goto out_free;
 			}
 			break;
@@ -519,8 +599,7 @@ ipmi_main(int argc, char ** argv,
 			memset(optarg, 'X', i);
 			break;
 		case 'E':
-			if ((tmp = getenv("IPMITOOL_PASSWORD")))
-			{
+			if ((tmp = getenv("IPMITOOL_PASSWORD"))) {
 				if (password)
 					free(password);
 				password = strdup(tmp);
@@ -529,8 +608,7 @@ ipmi_main(int argc, char ** argv,
 					goto out_free;
 				}
 			}
-			else if ((tmp = getenv("IPMI_PASSWORD")))
-			{
+			else if ((tmp = getenv("IPMI_PASSWORD"))) {
 				if (password)
 					free(password);
 				password = strdup(tmp);
@@ -558,22 +636,46 @@ ipmi_main(int argc, char ** argv,
 			authtype = str2val(optarg, ipmi_authtype_session_vals);
 			break;
 		case 't':
-			target_addr = (uint8_t)strtol(optarg, NULL, 0);
+			if (str2uchar(optarg, &target_addr) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-t'.");
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'b':
-			target_channel = (uint8_t)strtol(optarg, NULL, 0);
+			if (str2uchar(optarg, &target_channel) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-b'.");
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'T':
-			transit_addr = (uint8_t)strtol(optarg, NULL, 0);
+			if (str2uchar(optarg, &transit_addr) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-T'.");
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'B':
-			transit_channel = (uint8_t)strtol(optarg, NULL, 0);
+			if (str2uchar(optarg, &transit_channel) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-B'.");
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'l':
-			target_lun = (uint8_t)strtol(optarg, NULL, 0);
+			if (str2uchar(optarg, &target_lun) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-l'.");
+				rc = 1;
+				goto out_free;
+			}
 			break;
 		case 'm':
-			my_addr = (uint8_t)strtol(optarg, NULL, 0);
+			if (str2uchar(optarg, &my_addr) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-m'.");
+				rc = -1;
+				goto out_free;
+			}
 			break;
 		case 'e':
 			sol_escape_char = optarg[0];
@@ -585,6 +687,28 @@ ipmi_main(int argc, char ** argv,
 				goto out_free;
 			}
 			break;
+		case 'z':
+			if (str2ushort(optarg, &my_long_packet_size) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-z'.");
+				rc = -1;
+				goto out_free;
+			}
+			break;
+		/* Retry and Timeout */
+		case 'R':
+			if (str2int(optarg, &retry) != 0 || retry < 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-R'.");
+				rc = -1;
+				goto out_free;
+			}
+			break;
+		case 'N':
+			if (str2uint(optarg, &timeout) != 0) {
+				lprintf(LOG_ERR, "Invalid parameter given or out of range for '-N'.");
+				rc = -1;
+				goto out_free;
+			}
+			break;
 #endif
 		default:
 			ipmi_option_usage(progname, cmdlist, intflist);
@@ -594,7 +718,7 @@ ipmi_main(int argc, char ** argv,
 
 	/* check for command before doing anything */
 	if (argc-optind > 0 &&
-	    strncmp(argv[optind], "help", 4) == 0) {
+			strncmp(argv[optind], "help", 4) == 0) {
 		ipmi_cmd_print(cmdlist);
 		rc = 0;
 		goto out_free;
@@ -609,7 +733,7 @@ ipmi_main(int argc, char ** argv,
 	 * then prompt the user.
 	 */
 	if (hostname != NULL && password == NULL &&
-	    (authtype != IPMI_SESSION_AUTHTYPE_NONE || authtype < 0)) {
+			(authtype != IPMI_SESSION_AUTHTYPE_NONE || authtype < 0)) {
 #ifdef HAVE_GETPASSPHRASE
 		tmp = getpassphrase("Password: ");
 #else
@@ -637,9 +761,21 @@ ipmi_main(int argc, char ** argv,
 		}
 	}
 
+	if (password != NULL && intfname != NULL) {
+		if (strcmp(intfname, "lan") == 0 && strlen(password) > 16) {
+			lprintf(LOG_ERR, "%s: password is longer than 16 bytes.", intfname);
+			rc = -1;
+			goto out_free;
+		} else if (strcmp(intfname, "lanplus") == 0 && strlen(password) > 20) {
+			lprintf(LOG_ERR, "%s: password is longer than 20 bytes.", intfname);
+			rc = -1;
+			goto out_free;
+		}
+	} /* if (password != NULL && intfname != NULL) */
+
 	/* load interface */
-	intf = ipmi_intf_load(intfname);
-	if (intf == NULL) {
+	ipmi_main_intf = ipmi_intf_load(intfname);
+	if (ipmi_main_intf == NULL) {
 		lprintf(LOG_ERR, "Error loading interface %s", intfname);
 		goto out_free;
 	}
@@ -649,65 +785,127 @@ ipmi_main(int argc, char ** argv,
 
 	/* run OEM setup if found */
 	if (oemtype != NULL &&
-	    ipmi_oem_setup(intf, oemtype) < 0) {
+	    ipmi_oem_setup(ipmi_main_intf, oemtype) < 0) {
 		lprintf(LOG_ERR, "OEM setup for \"%s\" failed", oemtype);
 		goto out_free;
 	}
 
 	/* set session variables */
 	if (hostname != NULL)
-		ipmi_intf_session_set_hostname(intf, hostname);
+		ipmi_intf_session_set_hostname(ipmi_main_intf, hostname);
 	if (username != NULL)
-		ipmi_intf_session_set_username(intf, username);
+		ipmi_intf_session_set_username(ipmi_main_intf, username);
 	if (password != NULL)
-		ipmi_intf_session_set_password(intf, password);
+		ipmi_intf_session_set_password(ipmi_main_intf, password);
 	if (kgkey != NULL)
-		ipmi_intf_session_set_kgkey(intf, kgkey);
+		ipmi_intf_session_set_kgkey(ipmi_main_intf, kgkey);
 	if (port > 0)
-		ipmi_intf_session_set_port(intf, port);
+		ipmi_intf_session_set_port(ipmi_main_intf, port);
 	if (authtype >= 0)
-		ipmi_intf_session_set_authtype(intf, (uint8_t)authtype);
+		ipmi_intf_session_set_authtype(ipmi_main_intf, (uint8_t)authtype);
 	if (privlvl > 0)
-		ipmi_intf_session_set_privlvl(intf, (uint8_t)privlvl);
+		ipmi_intf_session_set_privlvl(ipmi_main_intf, (uint8_t)privlvl);
 	else
-		ipmi_intf_session_set_privlvl(intf,
-		      IPMI_SESSION_PRIV_ADMIN);	/* default */
+		ipmi_intf_session_set_privlvl(ipmi_main_intf,
+				IPMI_SESSION_PRIV_ADMIN);	/* default */
+	/* Adding retry and timeout for interface that support it */
+	if (retry > 0)
+		ipmi_intf_session_set_retry(ipmi_main_intf, retry);
+	if (timeout > 0)
+		ipmi_intf_session_set_timeout(ipmi_main_intf, timeout);
 
-	ipmi_intf_session_set_lookupbit(intf, lookupbit);
-	ipmi_intf_session_set_sol_escape_char(intf, sol_escape_char);
-	ipmi_intf_session_set_cipher_suite_id(intf, cipher_suite_id);
+	ipmi_intf_session_set_lookupbit(ipmi_main_intf, lookupbit);
+	ipmi_intf_session_set_sol_escape_char(ipmi_main_intf, sol_escape_char);
+	ipmi_intf_session_set_cipher_suite_id(ipmi_main_intf, cipher_suite_id);
 
 	/* setup destination lun if given */
-	intf->target_lun = target_lun ;
+	ipmi_main_intf->target_lun = target_lun ;
 
 	/* setup destination channel if given */
-	intf->target_channel = target_channel ;
+	ipmi_main_intf->target_channel = target_channel ;
 
-	intf->devnum = devnum;
+	ipmi_main_intf->devnum = devnum;
 
 	/* setup IPMB local and target address if given */
-	intf->my_addr = my_addr ? : IPMI_BMC_SLAVE_ADDR;
-	if (target_addr > 0) {
-		/* need to open the interface first */
-		if (intf->open != NULL)
-			intf->open(intf);
-		intf->target_addr = target_addr;
+	if (my_addr) {
+		ipmi_main_intf->my_addr = my_addr;
+	} else {
+		/* Check if PICMG extension is available to use the function 
+		 * GetDeviceLocator to retreive i2c address PICMG hack to set 
+		 * right IPMB address, If extension is not supported, should 
+		 * not give any problems
+		 *  PICMG Extension Version 2.0 (PICMG 3.0 Revision 1.0 ATCA) to
+		 *  PICMG Extension Version 2.3 (PICMG 3.0 Revision 3.0 ATCA)
+		 */
 
-      if (transit_addr > 0) {
-         intf->transit_addr    = transit_addr;
-         intf->transit_channel = transit_channel;
-      }
-      else
-      {
-         intf->transit_addr = intf->my_addr;
-      }
+		/* First, check if PICMG extension is available and supported */
+		struct ipmi_rq req;
+		struct ipmi_rs *rsp;
+		char msg_data;
+		unsigned char version_accepted = 0;
+
+		lprintf(LOG_INFO, "Running PICMG GetDeviceLocator" );
+		memset(&req, 0, sizeof(req));
+		req.msg.netfn = IPMI_NETFN_PICMG;
+		req.msg.cmd = PICMG_GET_PICMG_PROPERTIES_CMD;
+		msg_data    = 0x00;
+		req.msg.data = &msg_data;
+		req.msg.data_len = 1;
+		msg_data = 0;
+
+		rsp = ipmi_main_intf->sendrecv(ipmi_main_intf, &req);
+		if (rsp && !rsp->ccode) {
+			if ( (rsp->data[0] == 0) &&
+					((rsp->data[1] & 0x0F) == PICMG_ATCA_MAJOR_VERSION) )	{
+				version_accepted = 1;
+				lprintf(LOG_INFO, "Discovered PICMG Extension %d.%d",
+						(rsp->data[1] & 0x0f), (rsp->data[1] >> 4));
+			}
+		}
+		
+		if (version_accepted == 1) {
+			lprintf(LOG_DEBUG, "Running PICMG GetDeviceLocator");
+			memset(&req, 0, sizeof(req));
+			req.msg.netfn = IPMI_NETFN_PICMG;
+			req.msg.cmd = PICMG_GET_ADDRESS_INFO_CMD;
+			msg_data    = 0x00;
+			req.msg.data = &msg_data;
+			req.msg.data_len = 1;
+			msg_data = 0;
+
+			rsp = ipmi_main_intf->sendrecv(ipmi_main_intf, &req);
+			if (rsp && !rsp->ccode) {
+				ipmi_main_intf->my_addr = rsp->data[2];
+				ipmi_main_intf->target_addr = ipmi_main_intf->my_addr;
+				lprintf(LOG_INFO, "Discovered IPMB address = 0x%x",
+						ipmi_main_intf->my_addr);
+			}
+		} else {
+			lprintf(LOG_INFO,
+					"No PICMG Extenstion discovered, keeping IPMB address 0x20");
+		}
+	}
+
+	if ( target_addr > 0  && (target_addr != my_addr) ) {
+		/* need to open the interface first */
+		if (ipmi_main_intf->open != NULL)
+			ipmi_main_intf->open(ipmi_main_intf);
+
+		ipmi_main_intf->target_addr = target_addr;
+
+		if (transit_addr > 0) {
+			ipmi_main_intf->transit_addr    = transit_addr;
+			ipmi_main_intf->transit_channel = transit_channel;
+		} else {
+			ipmi_main_intf->transit_addr = ipmi_main_intf->my_addr;
+		}
 		/* must be admin level to do this over lan */
-		ipmi_intf_session_set_privlvl(intf, IPMI_SESSION_PRIV_ADMIN);
+		ipmi_intf_session_set_privlvl(ipmi_main_intf, IPMI_SESSION_PRIV_ADMIN);
 	}
 
 	/* parse local SDR cache if given */
 	if (sdrcache != NULL) {
-		ipmi_sdr_list_cache_fromfile(intf, sdrcache);
+		ipmi_sdr_list_cache_fromfile(ipmi_main_intf, sdrcache);
 	}
 
 	/* Parse SEL OEM file if given */
@@ -715,22 +913,39 @@ ipmi_main(int argc, char ** argv,
 		ipmi_sel_oem_init(seloem);
 	}
 
-	intf->cmdlist = cmdlist;
+	/* Enable Big Buffer when requested */
+	ipmi_main_intf->channel_buf_size = 0;
+	if ( my_long_packet_size != 0 ) {
+		printf("Setting large buffer to %i\n", my_long_packet_size);
+		if (ipmi_kontronoem_set_large_buffer( ipmi_main_intf, my_long_packet_size ) == 0)
+		{
+			my_long_packet_set = 1;
+			ipmi_main_intf->channel_buf_size = my_long_packet_size;
+		}
+	}
+
+	ipmi_main_intf->cmdlist = cmdlist;
 
 	/* now we finally run the command */
 	if (argc-optind > 0)
-		rc = ipmi_cmd_run(intf, argv[optind], argc-optind-1, &(argv[optind+1]));
+		rc = ipmi_cmd_run(ipmi_main_intf, argv[optind], argc-optind-1,
+				&(argv[optind+1]));
 	else
-		rc = ipmi_cmd_run(intf, NULL, 0, NULL);
+		rc = ipmi_cmd_run(ipmi_main_intf, NULL, 0, NULL);
+
+	if (my_long_packet_set == 1) {
+		/* Restore defaults */
+		ipmi_kontronoem_set_large_buffer( ipmi_main_intf, 0 );
+	}
 
 	/* clean repository caches */
-	ipmi_cleanup(intf);
+	ipmi_cleanup(ipmi_main_intf);
 
 	/* call interface close function if available */
-	if (intf->opened > 0 && intf->close != NULL)
-		intf->close(intf);
+	if (ipmi_main_intf->opened > 0 && ipmi_main_intf->close != NULL)
+		ipmi_main_intf->close(ipmi_main_intf);
 
- out_free:
+	out_free:
 	log_halt();
 
 	if (intfname != NULL)
