@@ -42,18 +42,72 @@
 #include <ipmitool/ipmi_sensor.h>
 
 extern int verbose;
-
-#define SCANNING_DISABLED       0x40
+// Macro's for Reading the current sensor Data.
+#define SCANNING_DISABLED	0x40
 #define READING_UNAVAILABLE	0x20
+#define	INVALID_THRESHOLD	"Invalid Threshold data values. Cannot Set Threshold Data."
+// static
+int
+ipmi_sensor_get_sensor_reading_factors(
+	struct ipmi_intf * intf, 
+	struct sdr_record_full_sensor * sensor, 
+	uint8_t reading)
+{
+	struct ipmi_rq req;
+	struct ipmi_rs * rsp;
+	uint8_t req_data[2];
+
+	char id[17];
+
+	if (intf == NULL || sensor == NULL)
+		return -1;
+
+	memset(id, 0, sizeof(id));
+	memcpy(id, sensor->id_string, 16);
+
+	req_data[0] = sensor->cmn.keys.sensor_num;
+	req_data[1] = reading;
+
+	memset(&req, 0, sizeof(req));
+	req.msg.netfn = IPMI_NETFN_SE;
+	req.msg.cmd   = GET_SENSOR_FACTORS;
+	req.msg.data  = req_data;
+	req.msg.data_len = sizeof(req_data);
+
+	rsp = intf->sendrecv(intf, &req);
+
+	if (rsp == NULL) {
+		lprintf(LOG_ERR, "Error updating reading factor for sensor %s (#%02x)",
+			id, sensor->cmn.keys.sensor_num);
+		return -1;
+	} else if (rsp->ccode) {
+		return -1;
+	} else {
+		/* Update SDR copy with updated Reading Factors for this reading */
+		/* Note: 
+		 * The Format of the returned data is exactly as in the SDR definition (Little Endian Format), 
+		 * therefore we can use raw copy operation here. 
+		 * Note: rsp->data[0] would point to the next valid entry in the sampling table
+		 */
+		 // BUGBUG: uses 'hardcoded' length information from SDR Definition
+		memcpy(&sensor->mtol, &rsp->data[1], sizeof(sensor->mtol));
+		memcpy(&sensor->bacc, &rsp->data[3], sizeof(sensor->bacc));
+		return 0;
+	}
+
+}
 
 static
 struct ipmi_rs *
 ipmi_sensor_set_sensor_thresholds(struct ipmi_intf *intf,
 				  uint8_t sensor,
-				  uint8_t threshold, uint8_t setting)
+				  uint8_t threshold, uint8_t setting,
+				  uint8_t target, uint8_t lun)
 {
 	struct ipmi_rq req;
 	static struct sensor_set_thresh_rq set_thresh_rq;
+	struct ipmi_rs *rsp;
+	uint8_t save_addr;
 
 	memset(&set_thresh_rq, 0, sizeof (set_thresh_rq));
 	set_thresh_rq.sensor_num = sensor;
@@ -73,49 +127,32 @@ ipmi_sensor_set_sensor_thresholds(struct ipmi_intf *intf,
 	else
 		return NULL;
 
+	save_addr = intf->target_addr;
+	intf->target_addr = target;
+
 	memset(&req, 0, sizeof (req));
 	req.msg.netfn = IPMI_NETFN_SE;
 	req.msg.cmd = SET_SENSOR_THRESHOLDS;
 	req.msg.data = (uint8_t *) & set_thresh_rq;
 	req.msg.data_len = sizeof (set_thresh_rq);
 
-	return intf->sendrecv(intf, &req);
+	rsp = intf->sendrecv(intf, &req);
+	intf->target_addr = save_addr;
+	return rsp;
 }
 
 static int
-ipmi_sensor_print_full_discrete(struct ipmi_intf *intf,
-				struct sdr_record_full_sensor *sensor)
+ipmi_sensor_print_fc_discrete(struct ipmi_intf *intf,
+				struct sdr_record_common_sensor *sensor,
+				uint8_t sdr_record_type)
 {
-	char id[17];
-	char *unitstr = "discrete";
-	int validread = 1;
-	uint8_t val = 0;
-	struct ipmi_rs *rsp;
+	const char *id;
+	struct sensor_reading *sr;
 
-	if (sensor == NULL)
+	sr = ipmi_sdr_read_sensor_value(intf, sensor, sdr_record_type, 3);
+
+	if (sr == NULL) {
 		return -1;
-
-	memset(id, 0, sizeof (id));
-	memcpy(id, sensor->id_string, 16);
-
-	/*
-	 * Get current reading
-	 */
-	rsp = ipmi_sdr_get_sensor_reading_ipmb(intf,
-					       sensor->keys.sensor_num,
-					       sensor->keys.owner_id,
-					       sensor->keys.lun);
-	if (rsp == NULL) {
-		lprintf(LOG_ERR, "Error reading sensor %s (#%02x)",
-			id, sensor->keys.sensor_num);
-		return -1;
-	} else if (rsp->ccode > 0 || (rsp->data[1] & READING_UNAVAILABLE)) {
-		validread = 0;
-	} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
-	  	validread = 0;
-	} else {
-		/* convert RAW reading into units */
-		val = rsp->data[0];
 	}
 
 	if (csv_output) {
@@ -125,14 +162,20 @@ ipmi_sensor_print_full_discrete(struct ipmi_intf *intf,
 			/* output format
 			 *   id value units status thresholds....
 			 */
-			printf("%-16s ", id);
-			if (validread) {
-				printf("| 0x%-8x | %-10s | 0x%02x%02x",
-				       val,
-				       unitstr, rsp->data[2], rsp->data[3]);
+			printf("%-16s ", sr->s_id);
+			if (sr->s_reading_valid) {
+				if (sr->s_has_analog_value) {
+					/* don't show discrete component */
+					printf("| %-10s | %-10s | %-6s",
+					       sr->s_a_str, sr->s_a_units, "ok");
+				} else {
+					printf("| 0x%-8x | %-10s | 0x%02x%02x",
+					       sr->s_reading, "discrete",
+					       sr->s_data2, sr->s_data3);
+				}
 			} else {
 				printf("| %-10s | %-10s | %-6s",
-				       "na", unitstr, "na");
+				       "na", "discrete", "na");
 			}
 			printf("| %-10s| %-10s| %-10s| %-10s| %-10s| %-10s",
 			       "na", "na", "na", "na", "na", "na");
@@ -140,153 +183,110 @@ ipmi_sensor_print_full_discrete(struct ipmi_intf *intf,
 			printf("\n");
 		} else {
 			printf("Sensor ID              : %s (0x%x)\n",
-			       id, sensor->keys.sensor_num);
+			       sr->s_id, sensor->keys.sensor_num);
 			printf(" Entity ID             : %d.%d\n",
 			       sensor->entity.id, sensor->entity.instance);
 			printf(" Sensor Type (Discrete): %s\n",
 			       ipmi_sdr_get_sensor_type_desc(sensor->sensor.
 							     type));
-         if( validread )
-         {
-            ipmi_sdr_print_discrete_state("States Asserted",
-												      sensor->sensor.type,
-												      sensor->event_type,
-												      rsp->data[2],
-												      rsp->data[3]);
+			if( sr->s_reading_valid )
+			{
+				if (sr->s_has_analog_value) {
+					printf(" Sensor Reading        : %s %s\n", sr->s_a_str, sr->s_a_units);
+				}
+				ipmi_sdr_print_discrete_state("States Asserted",
+							sensor->sensor.type,
+							sensor->event_type,
+							sr->s_data2,
+							sr->s_data3);
 				printf("\n");
-         }
-		}
+			} else {
+			   printf(" Unable to read sensor: Device Not Present\n\n");
+			}
+	   }
 	}
 
-	return (validread ? 0 : -1 );
+	return (sr->s_reading_valid ? 0 : -1 );
+}
+
+static void
+print_thresh_setting(struct sdr_record_full_sensor *full,
+			 uint8_t thresh_is_avail, uint8_t setting,
+			 const char *field_sep,
+			 const char *analog_fmt,
+			 const char *discrete_fmt,
+			 const char *na_fmt)
+{
+	printf("%s", field_sep);
+	if (!thresh_is_avail) {
+		printf(na_fmt, "na");
+		return;
+	}
+	if (full && !UNITS_ARE_DISCRETE(&full->cmn)) {
+		printf(analog_fmt, sdr_convert_sensor_reading (full, setting));
+	} else {
+		printf(discrete_fmt, setting);
+	}
 }
 
 static int
-ipmi_sensor_print_full_analog(struct ipmi_intf *intf,
-			      struct sdr_record_full_sensor *sensor)
+ipmi_sensor_print_fc_threshold(struct ipmi_intf *intf,
+			      struct sdr_record_common_sensor *sensor,
+			      uint8_t sdr_record_type)
 {
-	char unitstr[16], id[17];
-	int i = 0, validread = 1, thresh_available = 1;
-	double val = 0.0;
+	int thresh_available = 1;
 	struct ipmi_rs *rsp;
-	char *status = NULL;
+	struct sensor_reading *sr;
 
-	if (sensor == NULL)
-		return -1;
+	sr = ipmi_sdr_read_sensor_value(intf, sensor, sdr_record_type, 3);
 
-	memset(id, 0, sizeof (id));
-	memcpy(id, sensor->id_string, 16);
-
-	/* only handle linear and linearized sensors (for now) */
-	if (sensor->linearization >= SDR_SENSOR_L_NONLINEAR) {
-		printf("sensor %s non-linear!\n", id);
+	if (sr == NULL) {
 		return -1;
 	}
 
-	/*
-	 * Get current reading
-	 */
-	rsp = ipmi_sdr_get_sensor_reading_ipmb(intf,
-					       sensor->keys.sensor_num,
-					       sensor->keys.owner_id,
-					       sensor->keys.lun);
-	if (rsp == NULL) {
-		lprintf(LOG_ERR, "Error reading sensor %s (#%02x)",
-			id, sensor->keys.sensor_num);
-		return -1;
-	} else if (rsp->ccode || (rsp->data[1] & READING_UNAVAILABLE)) {
-		validread = 0;
-	} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
-		validread = 0;
-	} else {
-		/* convert RAW reading into units */
-		val = (rsp->data[0] > 0)
-		    ? sdr_convert_sensor_reading(sensor, rsp->data[0])
-		    : 0;
-		status = (char *) ipmi_sdr_get_status(sensor, rsp->data[2]);
-	}
-
-	/*
-	 * Figure out units
-	 */
-	memset(unitstr, 0, sizeof (unitstr));
-	switch (sensor->unit.modifier) {
-	case 2:
-		i += snprintf(unitstr, sizeof (unitstr), "%s * %s",
-			      unit_desc[sensor->unit.type.base],
-			      unit_desc[sensor->unit.type.modifier]);
-		break;
-	case 1:
-		i += snprintf(unitstr, sizeof (unitstr), "%s/%s",
-			      unit_desc[sensor->unit.type.base],
-			      unit_desc[sensor->unit.type.modifier]);
-		break;
-	case 0:
-	default:
-		i += snprintf(unitstr, sizeof (unitstr), "%s",
-			      unit_desc[sensor->unit.type.base]);
-		break;
-	}
+	const char *thresh_status = ipmi_sdr_get_thresh_status(sr, "ns");
 
 	/*
 	 * Get sensor thresholds
 	 */
-	rsp = ipmi_sdr_get_sensor_thresholds(intf, sensor->keys.sensor_num);
-	if (rsp == NULL)
+	rsp = ipmi_sdr_get_sensor_thresholds(intf,
+				sensor->keys.sensor_num, sensor->keys.owner_id,
+				sensor->keys.lun, sensor->keys.channel);
+
+	if ((rsp == NULL) || (rsp->ccode > 0) || (rsp->data_len == 0))
 		thresh_available = 0;
 
 	if (csv_output) {
-		/* NOT IPMLEMENTED */
+		/* NOT IMPLEMENTED */
 	} else {
 		if (verbose == 0) {
 			/* output format
 			 *   id value units status thresholds....
 			 */
-			printf("%-16s ", id);
-			if (validread) {
-				printf("| %-10.3f | %-10s | %-6s",
-				       val, unitstr, status ? : "");
+			printf("%-16s ", sr->s_id);
+			if (sr->s_reading_valid) {
+				if (sr->s_has_analog_value)
+					printf("| %-10.3f | %-10s | %-6s",
+					       sr->s_a_val, sr->s_a_units, thresh_status);
+				else
+					printf("| 0x%-8x | %-10s | %-6s",
+					       sr->s_reading, sr->s_a_units, thresh_status);
 			} else {
 				printf("| %-10s | %-10s | %-6s",
-				       "na", unitstr, "na");
+				       "na", sr->s_a_units, "na");
 			}
-			if (thresh_available) {
-				if (rsp->data[0] & LOWER_NON_RECOV_SPECIFIED)
-					printf("| %-10.3f",
-					       sdr_convert_sensor_reading
-					       (sensor, rsp->data[3]));
-				else
-					printf("| %-10s", "na");
-				if (rsp->data[0] & LOWER_CRIT_SPECIFIED)
-					printf("| %-10.3f",
-					       sdr_convert_sensor_reading
-					       (sensor, rsp->data[2]));
-				else
-					printf("| %-10s", "na");
-				if (rsp->data[0] & LOWER_NON_CRIT_SPECIFIED)
-					printf("| %-10.3f",
-					       sdr_convert_sensor_reading
-					       (sensor, rsp->data[1]));
-				else
-					printf("| %-10s", "na");
-				if (rsp->data[0] & UPPER_NON_CRIT_SPECIFIED)
-					printf("| %-10.3f",
-					       sdr_convert_sensor_reading
-					       (sensor, rsp->data[4]));
-				else
-					printf("| %-10s", "na");
-				if (rsp->data[0] & UPPER_CRIT_SPECIFIED)
-					printf("| %-10.3f",
-					       sdr_convert_sensor_reading
-					       (sensor, rsp->data[5]));
-				else
-					printf("| %-10s", "na");
-				if (rsp->data[0] & UPPER_NON_RECOV_SPECIFIED)
-					printf("| %-10.3f",
-					       sdr_convert_sensor_reading
-					       (sensor, rsp->data[6]));
-				else
-					printf("| %-10s", "na");
+			if (thresh_available && sr->full) {
+#define PTS(bit, dataidx) {						\
+	print_thresh_setting(sr->full, rsp->data[0] & (bit),  		\
+	    rsp->data[(dataidx)], "| ", "%-10.3f", "0x-8x", "%-10s");	\
+}
+				PTS(LOWER_NON_RECOV_SPECIFIED,	3);
+				PTS(LOWER_CRIT_SPECIFIED,	2);
+				PTS(LOWER_NON_CRIT_SPECIFIED,	1);
+				PTS(UPPER_NON_CRIT_SPECIFIED,	4);
+				PTS(UPPER_CRIT_SPECIFIED,	5);
+				PTS(UPPER_NON_RECOV_SPECIFIED,	6);
+#undef PTS
 			} else {
 				printf
 				    ("| %-10s| %-10s| %-10s| %-10s| %-10s| %-10s",
@@ -296,84 +296,72 @@ ipmi_sensor_print_full_analog(struct ipmi_intf *intf,
 			printf("\n");
 		} else {
 			printf("Sensor ID              : %s (0x%x)\n",
-			       id, sensor->keys.sensor_num);
+			       sr->s_id, sensor->keys.sensor_num);
 
 			printf(" Entity ID             : %d.%d\n",
 			       sensor->entity.id, sensor->entity.instance);
 
-			printf(" Sensor Type (Analog)  : %s\n",
+			printf(" Sensor Type (Threshold)  : %s\n",
 			       ipmi_sdr_get_sensor_type_desc(sensor->sensor.
 							     type));
 
 			printf(" Sensor Reading        : ");
-			if (validread) {
-				uint16_t raw_tol = __TO_TOL(sensor->mtol);
-				double tol =
-				    sdr_convert_sensor_tolerance(sensor,
-							       raw_tol);
-				printf("%.*f (+/- %.*f) %s\n",
-				       (val == (int) val) ? 0 : 3, val,
-				       (tol == (int) tol) ? 0 : 3, tol,
-				       unitstr);
-				printf(" Status                : %s\n",
-				       status ? : "");
+			if (sr->s_reading_valid) {
+				if (sr->full) {
+					uint16_t raw_tol = __TO_TOL(sr->full->mtol);
+					if (sr->s_has_analog_value) {
+						double tol =
+						    sdr_convert_sensor_tolerance(sr->full,
+									       raw_tol);
+						printf("%.*f (+/- %.*f) %s\n",
+						       (sr->s_a_val == (int)
+						       sr->s_a_val) ? 0 : 3,
+						       sr->s_a_val,
+						       (tol == (int) tol) ? 0 : 3, tol,
+						       sr->s_a_units);
+					} else {
+						printf("0x%x (+/- 0x%x) %s\n",
+						       sr->s_reading,
+						       raw_tol,
+						       sr->s_a_units);
+					}
+				} else {
+					printf("0x%x %s\n", sr->s_reading,
+					sr->s_a_units);
+				}
+				printf(" Status                : %s\n", thresh_status);
 
 				if (thresh_available) {
-					if (rsp->
-					    data[0] & LOWER_NON_RECOV_SPECIFIED)
-						printf
-						    (" Lower Non-Recoverable : %.3f\n",
-						     sdr_convert_sensor_reading
-						     (sensor, rsp->data[3]));
-					else
-						printf
-						    (" Lower Non-Recoverable : na\n");
-					if (rsp->data[0] & LOWER_CRIT_SPECIFIED)
-						printf
-						    (" Lower Critical        : %.3f\n",
-						     sdr_convert_sensor_reading
-						     (sensor, rsp->data[2]));
-					else
-						printf
-						    (" Lower Critical        : na\n");
-					if (rsp->
-					    data[0] & LOWER_NON_CRIT_SPECIFIED)
-						printf
-						    (" Lower Non-Critical    : %.3f\n",
-						     sdr_convert_sensor_reading
-						     (sensor, rsp->data[1]));
-					else
-						printf
-						    (" Lower Non-Critical    : na\n");
-					if (rsp->
-					    data[0] & UPPER_NON_CRIT_SPECIFIED)
-						printf
-						    (" Upper Non-Critical    : %.3f\n",
-						     sdr_convert_sensor_reading
-						     (sensor, rsp->data[4]));
-					else
-						printf
-						    (" Upper Non-Critical    : na\n");
-					if (rsp->data[0] & UPPER_CRIT_SPECIFIED)
-						printf
-						    (" Upper Critical        : %.3f\n",
-						     sdr_convert_sensor_reading
-						     (sensor, rsp->data[5]));
-					else
-						printf
-						    (" Upper Critical        : na\n");
-					if (rsp->
-					    data[0] & UPPER_NON_RECOV_SPECIFIED)
-						printf
-						    (" Upper Non-Recoverable : %.3f\n",
-						     sdr_convert_sensor_reading
-						     (sensor, rsp->data[6]));
-					else
-						printf
-						    (" Upper Non-Recoverable : na\n");
+					if (sr->full) {
+#define PTS(bit, dataidx, str) { 			\
+print_thresh_setting(sr->full, rsp->data[0] & (bit),	\
+		     rsp->data[(dataidx)], 		\
+		    (str), "%.3f\n", "0x%x\n", "%s\n"); \
+}
+
+						PTS(LOWER_NON_RECOV_SPECIFIED,	3, " Lower Non-Recoverable : ");
+						PTS(LOWER_CRIT_SPECIFIED,	2, " Lower Critical        : ");
+						PTS(LOWER_NON_CRIT_SPECIFIED,	1, " Lower Non-Critical    : ");
+						PTS(UPPER_NON_CRIT_SPECIFIED,	4, " Upper Non-Critical    : ");
+						PTS(UPPER_CRIT_SPECIFIED,	5, " Upper Critical        : ");
+						PTS(UPPER_NON_RECOV_SPECIFIED,	6, " Upper Non-Recoverable : ");
+#undef PTS
+
+					}
+					ipmi_sdr_print_sensor_hysteresis(sensor, sr->full,
+						sr->full ?  sr->full->threshold.hysteresis.positive :
+						sr->compact->threshold.hysteresis.positive,
+						"Positive Hysteresis");
+
+					ipmi_sdr_print_sensor_hysteresis(sensor, sr->full,
+						sr->full ?  sr->full->threshold.hysteresis.negative :
+						sr->compact->threshold.hysteresis.negative,
+						"Negative Hysteresis");
+				} else {
+					printf(" Sensor Threshold Settings not available\n");
 				}
 			} else {
-				printf("Not Present\n");
+			  printf(" Unable to read sensor: Device Not Present\n\n");
 			}
 
 			ipmi_sdr_print_sensor_event_status(intf,
@@ -381,110 +369,36 @@ ipmi_sensor_print_full_analog(struct ipmi_intf *intf,
 							   sensor_num,
 							   sensor->sensor.type,
 							   sensor->event_type,
-							   ANALOG_SENSOR);
+							   ANALOG_SENSOR,
+							   sensor->keys.owner_id,
+							   sensor->keys.lun,
+							   sensor->keys.channel);
 			ipmi_sdr_print_sensor_event_enable(intf,
 							   sensor->keys.
 							   sensor_num,
 							   sensor->sensor.type,
 							   sensor->event_type,
-							   ANALOG_SENSOR);
+							   ANALOG_SENSOR,
+							   sensor->keys.owner_id,
+							   sensor->keys.lun,
+							   sensor->keys.channel);
 
 			printf("\n");
 		}
 	}
 
-	return (validread ? 0 : -1 );
+	return (sr->s_reading_valid ? 0 : -1 );
 }
 
 int
-ipmi_sensor_print_full(struct ipmi_intf *intf,
-		       struct sdr_record_full_sensor *sensor)
+ipmi_sensor_print_fc(struct ipmi_intf *intf,
+		       struct sdr_record_common_sensor *sensor,
+			uint8_t sdr_record_type)
 {
-	if (sensor->unit.analog != 3)
-		return ipmi_sensor_print_full_analog(intf, sensor);
+	if (IS_THRESHOLD_SENSOR(sensor))
+		return ipmi_sensor_print_fc_threshold(intf, sensor, sdr_record_type);
 	else
-		return ipmi_sensor_print_full_discrete(intf, sensor);
-}
-
-int
-ipmi_sensor_print_compact(struct ipmi_intf *intf,
-			  struct sdr_record_compact_sensor *sensor)
-{
-	char id[17];
-	char *unitstr = "discrete";
-	int validread = 1;
-	uint8_t val = 0;
-	struct ipmi_rs *rsp;
-
-	if (sensor == NULL)
-		return -1;
-
-	memset(id, 0, sizeof (id));
-	memcpy(id, sensor->id_string, 16);
-
-	/*
-	 * Get current reading
-	 */
-	rsp = ipmi_sdr_get_sensor_reading_ipmb(intf,
-					       sensor->keys.sensor_num,
-					       sensor->keys.owner_id,
-					       sensor->keys.lun);
-	if (rsp == NULL) {
-		lprintf(LOG_ERR, "Error reading sensor %s (#%02x)",
-			id, sensor->keys.sensor_num);
-		return -1;
-	} else if (rsp->ccode || (rsp->data[1] & READING_UNAVAILABLE)) {
-		validread = 0;
-	} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
-		validread = 0;
-	} else {
-		/* convert RAW reading into units */
-		val = rsp->data[0];
-	}
-
-	if (csv_output) {
-		/* NOT IMPLEMENTED */
-	} else {
-		if (!verbose) {
-			/* output format
-			 *   id value units status thresholds....
-			 */
-			printf("%-16s ", id);
-
-			if (validread) {
-				printf("| 0x%-8x | %-10s | 0x%02x%02x",
-				       val, unitstr,
-				       rsp->data[2], rsp->data[3]);
-			} else {
-				printf("| %-10s | %-10s | %-6s",
-				       "na", unitstr, "na");
-			}
-
-			printf("| %-10s| %-10s| %-10s| %-10s| %-10s| %-10s",
-			       "na", "na", "na", "na", "na", "na");
-			printf("\n");
-		} else {
-			printf("Sensor ID              : %s (0x%x)\n",
-			       id, sensor->keys.sensor_num);
-			printf(" Entity ID             : %d.%d\n",
-			       sensor->entity.id, sensor->entity.instance);
-			printf(" Sensor Type (Discrete): %s\n",
-			       ipmi_sdr_get_sensor_type_desc(sensor->sensor.
-							     type));
-
-         if(validread)
-         {
-            ipmi_sdr_print_discrete_state("States Asserted",
-						      						sensor->sensor.type,
-												      sensor->event_type,
-												      rsp->data[2],
-												      rsp->data[3]);
-            printf("\n");
-         }
-		}
-	}
-
-	return (validread ? 0 : -1 );
+		return ipmi_sensor_print_fc_discrete(intf, sensor, sdr_record_type);
 }
 
 static int
@@ -496,14 +410,13 @@ ipmi_sensor_list(struct ipmi_intf *intf)
 
 	lprintf(LOG_DEBUG, "Querying SDR for sensor list");
 
-	itr = ipmi_sdr_start(intf);
+	itr = ipmi_sdr_start(intf, 0);
 	if (itr == NULL) {
 		lprintf(LOG_ERR, "Unable to open SDR for reading");
 		return -1;
 	}
 
 	while ((header = ipmi_sdr_get_next_header(intf, itr)) != NULL) {
-		int r = 0;
 		uint8_t *rec;
 
 		rec = ipmi_sdr_get_record(intf, header, itr);
@@ -514,22 +427,19 @@ ipmi_sensor_list(struct ipmi_intf *intf)
 
 		switch (header->type) {
 		case SDR_RECORD_TYPE_FULL_SENSOR:
-			r = ipmi_sensor_print_full(intf,
-						   (struct
-						    sdr_record_full_sensor *)
-						   rec);
-			break;
 		case SDR_RECORD_TYPE_COMPACT_SENSOR:
-			r = ipmi_sensor_print_compact(intf,
-						      (struct
-						       sdr_record_compact_sensor
-						       *) rec);
+			ipmi_sensor_print_fc(intf,
+						   (struct
+						    sdr_record_common_sensor *)
+						   rec,
+						   header->type);
 			break;
 		}
 		free(rec);
 
-		/* save any errors */
-		rc = (r == 0) ? rc : r;
+		/* fix for CR6604909: */
+		/* mask failure of individual reads in sensor list command */
+		/* rc = (r == 0) ? rc : r; */
 	}
 
 	ipmi_sdr_end(intf, itr);
@@ -549,11 +459,13 @@ static const struct valstr threshold_vals[] = {
 
 static int
 __ipmi_sensor_set_threshold(struct ipmi_intf *intf,
-			    uint8_t num, uint8_t mask, uint8_t setting)
+			    uint8_t num, uint8_t mask, uint8_t setting,
+			    uint8_t target, uint8_t lun)
 {
 	struct ipmi_rs *rsp;
 
-	rsp = ipmi_sensor_set_sensor_thresholds(intf, num, mask, setting);
+	rsp = ipmi_sensor_set_sensor_thresholds(intf, num, mask, setting,
+				  target, lun);
 
 	if (rsp == NULL) {
 		lprintf(LOG_ERR, "Error setting threshold");
@@ -568,6 +480,28 @@ __ipmi_sensor_set_threshold(struct ipmi_intf *intf,
 	return 0;
 }
 
+static uint8_t
+__ipmi_sensor_threshold_value_to_raw(struct sdr_record_full_sensor *full, double value)
+{
+	if (!UNITS_ARE_DISCRETE(&full->cmn)) { /* Has an analog reading */
+		/* Has an analog reading and supports mx+b */
+		return sdr_convert_sensor_value_to_raw(full, value);
+	}
+	else {
+		/* Does not have an analog reading and/or does not support mx+b */
+		if (value > 255) {
+			return 255;
+		}
+		else if (value < 0) {
+			return 0;
+		}
+		else {
+			return (uint8_t )value;
+		}
+	}
+}
+
+
 static int
 ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 {
@@ -576,6 +510,9 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 	double setting1 = 0.0, setting2 = 0.0, setting3 = 0.0;
 	int allUpper = 0, allLower = 0;
 	int ret = 0;
+	struct ipmi_rs *rsp;
+	int i =0;
+	double val[10] = {0};
 
 	struct sdr_record_list *sdr;
 
@@ -669,76 +606,176 @@ ipmi_sensor_set_threshold(struct ipmi_intf *intf, int argc, char **argv)
 		return -1;
 	}
 
+	if (!IS_THRESHOLD_SENSOR(sdr->record.common)) {
+		lprintf(LOG_ERR, "Invalid sensor event type %02x", sdr->record.common->event_type);
+		return -1;
+	}
+
+
 	if (allUpper) {
 		settingMask = UPPER_NON_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting1);
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting1));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting1),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 
 		settingMask = UPPER_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting2);
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting2));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting2),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 
 		settingMask = UPPER_NON_RECOV_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting3);
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting3));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting3),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 	} else if (allLower) {
 		settingMask = LOWER_NON_RECOV_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting1);
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting1));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting1),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 
 		settingMask = LOWER_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting2);
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting2));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting2),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 
 		settingMask = LOWER_NON_CRIT_SPECIFIED;
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting3);
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting3));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting3),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 	} else {
+
+	/*
+ 	 * Current implementation doesn't check for the valid setting of upper non critical and other thresholds.
+ 	 * In the below logic:
+ 	 * 	Get all the current reading of the sensor i.e. unc, uc, lc,lnc.
+ 	 * 	Validate the values given by the user.
+ 	 * 	If the values are not correct, then popup with the Error message and return.
+ 	 */
+	/*
+	 * Get current reading
+	 */
+		rsp = ipmi_sdr_get_sensor_reading_ipmb(intf,
+					       sdr->record.common->keys.sensor_num,
+					       sdr->record.common->keys.owner_id,
+					       sdr->record.common->keys.lun,sdr->record.common->keys.channel);
+		rsp = ipmi_sdr_get_sensor_thresholds(intf,
+						sdr->record.common->keys.sensor_num,
+						sdr->record.common->keys.owner_id,
+						sdr->record.common->keys.lun,
+						sdr->record.common->keys.channel);
+		if ((rsp == NULL) || (rsp->ccode > 0)) {
+			lprintf(LOG_ERR, "Sensor data record not found!");
+				return -1;
+		}
+		for(i=1;i<=6;i++) {
+			val[i] = sdr_convert_sensor_reading(sdr->record.full, rsp->data[i]);
+			if(val[i] < 0)
+				val[i] = 0;
+		}
+		/* Check for the valid Upper non recovarable Value.*/
+		if( (settingMask & UPPER_NON_RECOV_SPECIFIED) ) {
+
+			if( (rsp->data[0] & UPPER_NON_RECOV_SPECIFIED) &&
+				(( (rsp->data[0] & UPPER_CRIT_SPECIFIED) && ( setting1 <= val[5])) ||
+					( (rsp->data[0] & UPPER_NON_CRIT_SPECIFIED) && ( setting1 <= val[4]))) )
+			{
+				lprintf(LOG_ERR, INVALID_THRESHOLD);
+				return -1;
+			}
+		} else if( (settingMask & UPPER_CRIT_SPECIFIED) ) { 		/* Check for the valid Upper critical Value.*/
+			if( (rsp->data[0] & UPPER_CRIT_SPECIFIED) &&
+				(((rsp->data[0] & UPPER_NON_RECOV_SPECIFIED)&& ( setting1 >= val[6])) ||
+				((rsp->data[0] & UPPER_NON_CRIT_SPECIFIED)&&( setting1 <= val[4]))) )
+			{
+				lprintf(LOG_ERR, INVALID_THRESHOLD);
+				return -1;
+			}
+		} else if( (settingMask & UPPER_NON_CRIT_SPECIFIED) ) {  		/* Check for the valid Upper non critical Value.*/
+			if( (rsp->data[0] & UPPER_NON_CRIT_SPECIFIED) &&
+				(((rsp->data[0] & UPPER_NON_RECOV_SPECIFIED)&&( setting1 >= val[6])) ||
+				((rsp->data[0] & UPPER_CRIT_SPECIFIED)&&( setting1 >= val[5])) ||
+				((rsp->data[0] & LOWER_NON_CRIT_SPECIFIED)&&( setting1 <= val[1]))) )
+			{
+				lprintf(LOG_ERR, INVALID_THRESHOLD);
+				return -1;
+			}
+		} else if( (settingMask & LOWER_NON_CRIT_SPECIFIED) ) {		/* Check for the valid lower non critical Value.*/
+			if( (rsp->data[0] & LOWER_NON_CRIT_SPECIFIED) &&
+				(((rsp->data[0] & LOWER_CRIT_SPECIFIED)&&( setting1 <= val[2])) ||
+				((rsp->data[0] & LOWER_NON_RECOV_SPECIFIED)&&( setting1 <= val[3]))||
+				((rsp->data[0] & UPPER_NON_CRIT_SPECIFIED)&&( setting1 >= val[4]))) )
+			{
+				lprintf(LOG_ERR, INVALID_THRESHOLD);
+				return -1;
+			}
+		} else if( (settingMask & LOWER_CRIT_SPECIFIED) ) {		/* Check for the valid lower critical Value.*/
+			if( (rsp->data[0] & LOWER_CRIT_SPECIFIED) &&
+				(((rsp->data[0] & LOWER_NON_CRIT_SPECIFIED)&&( setting1 >= val[1])) ||
+				((rsp->data[0] & LOWER_NON_RECOV_SPECIFIED)&&( setting1 <= val[3]))) )
+			{
+				lprintf(LOG_ERR, INVALID_THRESHOLD);
+				return -1;
+			}
+		} else if( (settingMask & LOWER_NON_RECOV_SPECIFIED) ) {		/* Check for the valid lower non recovarable Value.*/
+			if( (rsp->data[0] & LOWER_NON_RECOV_SPECIFIED) &&
+				(((rsp->data[0] & LOWER_NON_CRIT_SPECIFIED)&&( setting1 >= val[1])) ||
+				((rsp->data[0] & LOWER_CRIT_SPECIFIED)&&( setting1 >= val[2]))) )
+			{
+				lprintf(LOG_ERR, INVALID_THRESHOLD);
+				return -1;
+			}
+		} else {			/* None of this Then Return with error messages.*/
+			lprintf(LOG_ERR, INVALID_THRESHOLD);
+			return -1;
+		}
+
+
 		printf("Setting sensor \"%s\" %s threshold to %.3f\n",
 		       sdr->record.full->id_string,
 		       val2str(settingMask, threshold_vals), setting1);
 
 		ret = __ipmi_sensor_set_threshold(intf,
-						  sdr->record.full->keys.
+						  sdr->record.common->keys.
 						  sensor_num, settingMask,
-						  sdr_convert_sensor_value_to_raw
-						  (sdr->record.full, setting1));
+						  __ipmi_sensor_threshold_value_to_raw(sdr->record.full, setting1),
+						  sdr->record.common->keys.owner_id,
+						  sdr->record.common->keys.lun);
 	}
 
 	return ret;
@@ -748,9 +785,7 @@ static int
 ipmi_sensor_get_reading(struct ipmi_intf *intf, int argc, char **argv)
 {
 	struct sdr_record_list *sdr;
-	struct ipmi_rs *rsp;
 	int i, rc=0;
-	double val = 0.0;
 
 	if (argc < 1 || strncmp(argv[0], "help", 4) == 0) {
 		lprintf(LOG_NOTICE, "sensor reading <id> ... [id]");
@@ -769,42 +804,34 @@ ipmi_sensor_get_reading(struct ipmi_intf *intf, int argc, char **argv)
 
 		switch (sdr->type) {
 		case SDR_RECORD_TYPE_FULL_SENSOR:
-			if (sdr->record.full->linearization >= SDR_SENSOR_L_NONLINEAR) {
-				lprintf(LOG_ERR, "Sensor \"%s\" non-linear!", argv[i]);
-				continue;
-			}
-			rsp = ipmi_sdr_get_sensor_reading_ipmb(intf,
-							       sdr->record.full->keys.sensor_num,
-							       sdr->record.full->keys.owner_id,
-							       sdr->record.full->keys.lun);
-			if (rsp == NULL) {
-				lprintf(LOG_ERR, "Error reading sensor \"%s\"", argv[i]);
+		case SDR_RECORD_TYPE_COMPACT_SENSOR:
+		{
+			struct sensor_reading *sr;
+			struct sdr_record_common_sensor	*sensor = sdr->record.common;
+			sr = ipmi_sdr_read_sensor_value(intf, sensor, sdr->type, 3);
+
+			if (sr == NULL) {
 				rc = -1;
 				continue;
-			} else if (rsp->ccode > 0) {
-				continue;
-			} else if (rsp->data[1] & READING_UNAVAILABLE) {
-				continue;
-			} else if (!(rsp->data[1] & SCANNING_DISABLED)) {
-				continue;
-			} else if (rsp->data[0] > 0) {
-				/* convert RAW reading into units */
-				val = sdr_convert_sensor_reading(sdr->record.full, rsp->data[0]);
-			} else {
-				val = 0.0;
 			}
 
+			if (!sr->full)
+				continue;
+
+			if (!sr->s_reading_valid)
+				continue;
+
+			if (!sr->s_has_analog_value) {
+				lprintf(LOG_ERR, "Sensor \"%s\" is a discrete sensor!", argv[i]);
+				continue;
+			}
 			if (csv_output)
-				printf("%s,%.*f\n", argv[i],
-				       (val == (int)val) ? 0 : 3, val);
+				printf("%s,%s\n", argv[i], sr->s_a_str);
 			else
-				printf("%-16s | %.*f\n", argv[i],
-				       (val == (int)val) ? 0 : 3, val);
+				printf("%-16s | %s\n", argv[i], sr->s_a_str);
 
 			break;
-
-		case SDR_RECORD_TYPE_COMPACT_SENSOR:
-			break;
+		}
 		default:
 			continue;
 		}
@@ -844,11 +871,9 @@ ipmi_sensor_get(struct ipmi_intf *intf, int argc, char **argv)
 		verbose = 1;
 		switch (sdr->type) {
 		case SDR_RECORD_TYPE_FULL_SENSOR:
-			r = ipmi_sensor_print_full(intf, sdr->record.full);
-			break;
 		case SDR_RECORD_TYPE_COMPACT_SENSOR:
-			r = ipmi_sensor_print_compact(intf,
-						      sdr->record.compact);
+			r = ipmi_sensor_print_fc(intf, sdr->record.common,
+						   sdr->type);
 			break;
 		case SDR_RECORD_TYPE_EVENTONLY_SENSOR:
 			r = ipmi_sdr_print_sensor_eventonly(intf,
