@@ -645,8 +645,8 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 		 * 7) An Invalid packet (one that doesn't match a request)
 		 * -------------------------------------------------------------------
 		 */
-		read_session_data(rsp, &offset, intf->session);
 
+		read_session_data(rsp, &offset, intf->session);
 
 		if (lanplus_has_valid_auth_code(rsp, intf->session) == 0)
 		{
@@ -714,44 +714,45 @@ ipmi_lan_poll_recv(struct ipmi_intf * intf)
 			/* Are we expecting this packet? */
 			entry = ipmi_req_lookup_entry(rsp->payload.ipmi_response.rq_seq,
 								rsp->payload.ipmi_response.cmd);
+
 			if (entry != NULL) {
 				lprintf(LOG_DEBUG+2, "IPMI Request Match found");
-				if (
-				      intf->target_addr != intf->my_addr 
-				      &&
-					   bridgePossible
-                  &&
-                  rsp->data_len 
-                  &&
-						rsp->payload.ipmi_response.cmd == 0x34
-					) 
+				if ( intf->target_addr != intf->my_addr &&
+				     bridgePossible &&
+				     rsp->data_len &&
+				     rsp->payload.ipmi_response.cmd == 0x34 &&
+				     (rsp->payload.ipmi_response.netfn == 0x06 ||
+				     rsp->payload.ipmi_response.netfn == 0x07) &&
+				     rsp->payload.ipmi_response.rs_lun == 0 )
 				{
-               /* Check completion code */
-               if (rsp->data[offset-1] == 0)
-               {
-					   lprintf(LOG_DEBUG, "Bridged command answer,"
-					         " waiting for next answer... ");
-      				ipmi_req_remove_entry(rsp->payload.ipmi_response.rq_seq,
-								rsp->payload.ipmi_response.cmd);
-                  ipmi_lan_poll_recv(intf);
-                  return;
+					/* Check completion code */
+					if (rsp->data[offset-1] == 0)
+					{
+						lprintf(LOG_DEBUG, "Bridged command answer,"
+						     " waiting for next answer... ");
+						ipmi_req_remove_entry(
+							rsp->payload.ipmi_response.rq_seq,
+							rsp->payload.ipmi_response.cmd);
+						return ipmi_lan_poll_recv(intf);
 					}
 					else
-					{		
-					   lprintf(LOG_DEBUG, "WARNING: Bridged "
-								"cmd ccode = 0x%02x",
-								rsp->data[offset-1]);
-               }
+					{
+						lprintf(LOG_DEBUG, "WARNING: Bridged "
+								   "cmd ccode = 0x%02x",
+								   rsp->data[offset-1]);
+					}
 
 					if (rsp->data_len &&
-						 rsp->payload.ipmi_response.cmd == 0x34) {
-                  memcpy(rsp->data, &rsp->data[offset],(rsp->data_len-offset)); 
-						printbuf(
-							&rsp->data[offset],
+					    rsp->payload.ipmi_response.cmd == 0x34) {
+						memcpy(rsp->data, &rsp->data[offset],
+							(rsp->data_len-offset));
+						if (verbose > 2)
+							printbuf( &rsp->data[offset],
 							(rsp->data_len-offset),
 							"bridge command response");
 					}
 				}
+
 				ipmi_req_remove_entry(rsp->payload.ipmi_response.rq_seq,
 								rsp->payload.ipmi_response.cmd);
 			} else {
@@ -1843,7 +1844,8 @@ ipmi_lanplus_build_v2x_msg(
 static struct ipmi_rq_entry *
 ipmi_lanplus_build_v2x_ipmi_cmd(
 								struct ipmi_intf * intf,
-								struct ipmi_rq * req)
+								struct ipmi_rq * req,
+								int isRetry)
 {
 	struct ipmi_v2_payload v2_payload;
 	struct ipmi_rq_entry * entry;
@@ -1856,7 +1858,8 @@ ipmi_lanplus_build_v2x_ipmi_cmd(
 	 */
 	static uint8_t curr_seq = 0;
 
-	curr_seq += 1;
+	if( isRetry == 0 )
+		curr_seq += 1;
 
 	if (curr_seq >= 64)
 		curr_seq = 0;
@@ -2087,14 +2090,21 @@ ipmi_lanplus_send_payload(
 	int                   try = 0;
 	int                   xmit = 1;
 	time_t                ltime;
+	uint32_t	      timeout;
 
 	if (!intf->opened && intf->open && intf->open(intf) < 0)
 		return NULL;
 
+	/*
+	 * The session timeout is initialized in the above interface open,
+	 * so it will only be valid after the open completes.
+	 */
+	timeout = session->timeout;
 	while (try < session->retry) {
-		ltime = time(NULL);
+		//ltime = time(NULL);
 
 		if (xmit) {
+			ltime = time(NULL);
 
 			if (payload->payload_type == IPMI_PAYLOAD_TYPE_IPMI)
 			{
@@ -2134,8 +2144,10 @@ ipmi_lanplus_send_payload(
 				}
 				else
 				{
+					int isRetry = ( try > 0 ? 1 : 0 );
+
 					lprintf(LOG_DEBUG+1, "BUILDING A v2 COMMAND");
-					entry = ipmi_lanplus_build_v2x_ipmi_cmd(intf, ipmi_request);
+					entry = ipmi_lanplus_build_v2x_ipmi_cmd(intf, ipmi_request, isRetry);
 				}
 
 				if (entry == NULL) {
@@ -2269,25 +2281,32 @@ ipmi_lanplus_send_payload(
 		else
 		{
 			rsp = ipmi_lan_poll_recv(intf);
+
+			/* Duplicate Request ccode most likely indicates a response to
+			   a previous retry. Ignore and keep polling. */
+			while ((rsp != NULL) && (rsp->ccode == 0xcf))
+			{
+				rsp = NULL;
+				rsp = ipmi_lan_poll_recv(intf);
+			}
+
 			if (rsp)
 				break;
 		}
 
-		xmit = ((time(NULL) - ltime) >= intf->session->timeout);
+		/* only timeout if time exceeds the timeout value */
+		xmit = ((time(NULL) - ltime) > timeout);
 
 		usleep(5000);
 
 		if (xmit) {
-			/* incremet session timeout each retry */
-			intf->session->timeout++;
+			/* increment session timeout by 1 second each retry */
+			timeout++;
 		}
 
 		try++;
 	}
 
-	/* Reset timeout after retry loop completes */
-	intf->session->timeout = IPMI_LAN_TIMEOUT;
-
 	/* IPMI messages are deleted under ipmi_lan_poll_recv() */
 	switch (payload->payload_type) {
 	case IPMI_PAYLOAD_TYPE_RMCP_OPEN_REQUEST:
@@ -2677,6 +2696,7 @@ ipmi_close_session_cmd(struct ipmi_intf * intf)
 
 	backupBridgePossible = bridgePossible;
 
+	intf->target_addr = IPMI_BMC_SLAVE_ADDR;
 	bridgePossible = 0;
 
 	bmc_session_lsbf = intf->session->v2_data.bmc_id;
@@ -3227,6 +3247,7 @@ ipmi_lanplus_close(struct ipmi_intf * intf)
 
 	intf->session = NULL;
 	intf->opened = 0;
+	intf->manufacturer_id = IPMI_OEM_UNKNOWN;
 	intf = NULL;
 }
 
@@ -3363,7 +3384,6 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 
 	intf->opened = 1;
 
-
 	/*
 	 *
 	 * Make sure the BMC supports IPMI v2 / RMCP+
@@ -3421,6 +3441,7 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	if (rc < 0)
 		goto fail;
 
+	intf->manufacturer_id = ipmi_get_oem(intf);
 	return intf->fd;
 
  fail:
